#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Github:       https://github.com/cruizperez
# Institution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: This script parses a table with protein IDs and searches
# these IDs in a SQL database to extract protein annotations.
# You can also include a column with the ids of your proteins (queries)
# so they will be included in the output_file (use query_col [int]).
########################################################################
"""
################################################################################
"""---0.0 Import Modules---"""
import sqlite3
import argparse, sys

################################################################################
"""---1.0 Define Functions---"""

def search_table(sql_database, database_table, input_list, outfile):
    """ Searches a table in a SQLite3 database and extracts the target_id annotation.
    
    Arguments:
        sql_database {string} -- SQLite3 database to use for searching.
        database_table {string} -- Table within SQLite3 database to search.
        input_list {list} -- List with (query_ids and target_ids) or just target_ids.
        
        outfile {string} -- File to write output_file
    """ 
    conn = sqlite3.connect(sql_database)
    cur = conn.cursor()
    if isinstance(input_list[0], tuple):
        query_present = True
    else:
        query_present = False
    # Check which columns to use for output_file
    if database_table == "swissprot" or database_table == "trembl":
        if query_present:
            col_names = ['#query_id', 'target_id', 'accession', 'name', 'ko', 'organism', 'taxonomy',
                        'function_go', 'compartment_go', 'process_go', 'interpro', 'pfam', 'ec_num']
        else:
            col_names = ['#target_id', 'accession', 'name', 'ko', 'organism', 'taxonomy',
                        'function_go', 'compartment_go', 'process_go', 'interpro', 'pfam', 'ec_num']
    elif database_table == "refseq":
        if query_present:
            col_names = ['#query_id', 'target_id', 'gene_name', 'taxonomy', 'ec_num', 'ko', 'ko_product']
        else:
            col_names = ['#target_id', 'gene_name', 'taxonomy', 'ec_num', 'KO', 'ko_product']
    # Search the DB and append results to Annotation_List
    with open(outfile, 'w') as output_file:
        output_file.write("{}\n".format("\t".join(col_names)))
        if query_present:
            for gene_id in input_list:
                cur.execute("SELECT * FROM " + database_table + " WHERE gene_id=?", (gene_id[1],))
                rows = cur.fetchall()
                if len(rows) > 0:
                    annotation = "\t".join(rows[0])
                    output_file.write("{}\t{}\n".format(gene_id[0], annotation))
                else:
                    continue
        else:
            for gene_id in input_list:
                cur.execute("SELECT * FROM " + database_table + " WHERE gene_id=?", (gene_id,))
                rows = cur.fetchall()
                if len(rows) > 0:
                    annotation = "\t".join(rows[0])
                    output_file.write("{}\n".format(annotation))
                else:
                    continue
    cur.close()
    conn.close()

def parse_input_table(input_file, query_col, target_col, table):
    """ Parses input file and returns query_ids and target_ids or just target_ids.
    
    Arguments:
        input_file {string} -- Tab-separated input file with blast results or just target ids.
        query_col {int} -- Column with query ids.
        target_col {int} -- Column with target ids.
    
    Returns:
        [list] -- List of (query_ids and target_ids) or just target_ids.
    """
    input_list = []
    with open(input_file, 'r') as infile:
        if table == "swissprot" or table  == "trembl":
            for line in infile:
                line = line.strip().split("\t")
                db_hit = line[target_col].split("|")[2]
                if query_col is not None:
                    input_query = line[query_col]
                    input_list.append((input_query, db_hit))
                else:
                    input_list.append(db_hit)
        else:
            for line in infile:
                line = line.strip().split("\t")
                db_hit = line[target_col]
                if query_col is not None:
                    input_query = line[query_col]
                    input_list.append((input_query, db_hit))
                else:
                    input_list.append(db_hit)
    return input_list


def search_ids_imported(sql_database, database_table, input_list):
    """ Searches a table in a SQLite3 database and extracts the target_id annotation.
    
    Arguments:
        sql_database {string} -- SQLite3 database to use for searching.
        database_table {string} -- Table within SQLite3 database to search.
        input_list {list} -- List with (query_ids and target_ids) or just target_ids.
    """ 
    conn = sqlite3.connect(sql_database)
    cur = conn.cursor()
    # Check which columns to use for output_file
    # Uniprot db fields:
    # protein_id accession product ko_number organism taxonomy function compartment process
    # RefSeq db fields:
    # protein_id product taxonomy ko_number ko_product
    # Search the DB and return results in list
    resulting_annotations = []
    for id_pair in input_list:
        if database_table == "refseq":
            gene_id = id_pair[1]
        else:
            gene_id = id_pair[1].split("|")[2]
        cur.execute("SELECT * FROM " + database_table + " WHERE gene_id=?", (gene_id,))
        rows = cur.fetchall()
        if len(rows) > 0:
            annotation = rows[0]
            annotation = list(annotation)
            annotation.insert(0, id_pair[0])
            resulting_annotations.append(annotation)
        else:
            continue
    cur.close()
    conn.close()
    return resulting_annotations


################################################################################
"""---3.0 Main Function---"""

def main():
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''This script parses a table with protein IDs and searches\n'''
            '''these IDs in a SQL database to extract protein annotations.\n'''
            '''You can also include a column with the ids of your proteins (queries)\n'''
            '''so they will be included in the output_file (use query_col [int]).\n'''
            '''Usage: ''' + sys.argv[0] + ''' -i [Input Table] -d [Database Name] -t [Table Name] --query_col [Query Column]\n'''
            '''Global mandatory parameters: -i [Input Table] -d [Database Name] -t [Table Name] --query_col [Query Column]\n'''
            '''Optional Database Parameters: See ''' + sys.argv[0] + ' -h')
    parser.add_argument('-i', '--input', dest='input_file', action='store', required=True,
                        help='Input tab-delimited table to parse, by default assumes no headers are present')
    parser.add_argument('-o', '--output_file', dest='outfile', action='store', required=True,
                        help='output_file tab-delimited table to store annotations')
    parser.add_argument('-d', '--database', dest='database', action='store', required=True,
                        help='SQL database where the annotations are stored')
    parser.add_argument('-t', '--table', dest='table', action='store', required=True,
                        help='Table within SQLite database to search in. Choose between "swissprot", "trembl" or "refseq"')
    parser.add_argument('--target_col', dest='target_col', action='store', required=False, type=int, default=1, 
                        help='Column with ids of database hits (targets) in the input file. By default 1, first column.')
    parser.add_argument('--query_col', dest='query_col', action='store', required=False, type=int,
                        help='Column with query ids in the input file, by default None, i.e. assumes only target ids are given.')
    args = parser.parse_args()

    input_file = args.input_file
    outfile = args.outfile
    database = args.database
    table = args.table.lower()
    target_col = args.target_col - 1
    query_col = args.query_col
    if query_col != None:
        query_col = query_col - 1

    # Read input table and extract ids.
    input_list = parse_input_table(input_file, query_col, target_col, table)
    search_table(database, table, input_list, outfile)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: Create SQLite database from a tab-delimited table.
Used to create annotation parsing databases.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
import sqlite3
from sys import exit
import wget
import gzip
from pathlib import Path

################################################################################
"""---1.0 Define Functions---"""
def create_swissprot_table(database, table):
    conn = sqlite3.connect(database)
    cursor = conn.cursor()
    cursor.execute("DROP TABLE IF EXISTS swissprot")
    cursor.execute('CREATE TABLE swissprot \
        (gene_id TEXT, accession TEXT, gene_name TEXT, ko_number TEXT, \
        organism TEXT, taxonomy TEXT, function TEXT, compartment TEXT, \
        process TEXT, interpro TEXT, pfam TEXT, ec_number TEXT)')
    with open(table, 'r') as swissprot:
        record_counter = 0
        records = []
        for line in swissprot:
            # Commit changes after 500000 records
            if record_counter == 500000:
                cursor.execute("begin")
                cursor.executemany('INSERT INTO swissprot VALUES(?, ?, ?, ?, ?, \
                    ?, ?, ?, ?, ?, ?, ?)', records)
                cursor.execute("commit")
                record_counter = 0
                records = []
            else:
                records.append(tuple(line.rstrip('\n').split("\t")))
                record_counter += 1
        # Commit remaining records
        if record_counter > 0:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO swissprot VALUES(?, ?, ?, ?, ?, \
                ?, ?, ?, ?, ?, ?, ?)', records)
            cursor.execute("commit")
        # Create index for faster access
        cursor.execute('CREATE INDEX swissprot_id ON swissprot (gene_id)')

def create_trembl_table(database, table):
    conn = sqlite3.connect(database)
    cursor = conn.cursor()
    cursor.execute("DROP TABLE IF EXISTS trembl")
    cursor.execute('CREATE TABLE trembl \
        (gene_id TEXT, accession TEXT, gene_name TEXT, ko_number TEXT, \
        organism TEXT, taxonomy TEXT, function TEXT, compartment TEXT, \
        process TEXT, interpro TEXT, pfam TEXT, ec_number)')
    with open(table, 'r') as trembl:
        record_counter = 0
        records = []
        for line in trembl:
            # Commit changes after 500000 records
            if record_counter == 500000:
                cursor.execute("begin")
                cursor.executemany('INSERT INTO trembl VALUES(?, ?, ?, ?, ?, \
                    ?, ?, ?, ?, ?, ?, ?)', records)
                cursor.execute("commit")
                record_counter = 0
                records = []
            else:
                records.append(tuple(line.rstrip('\n').split("\t")))
                record_counter += 1
        # Commit remaining records
        if record_counter > 0:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO trembl VALUES(?, ?, ?, ?, ?, \
                ?, ?, ?, ?, ?, ?, ?)', records)
            cursor.execute("commit")
        # Create index for faster access
        cursor.execute('CREATE INDEX trembl_id ON trembl (gene_id)')

def create_refseq_table(database, table):
    # Store protein ko annotations
    # Download refseq KO annotations
    print("Parsing RefSeq KO numbers")
    annotation_file = wget.download("http://enve-omics.ce.gatech.edu/data/public_microbeannotator/01.RefSeq_KEGG_Annotations.txt.gz",
                            out="refseq_ko_annotations.tar.gz")
    protein_annotations = {}
    with gzip.open(annotation_file, 'rt') as annotations:
        for line in annotations:
            line = line.rstrip('\n').split("\t")
            protein_annotations[line[0]] = [line[1], line[2]]
    Path(annotation_file).unlink()
    print("Done")
    # Connect to db and create table
    print("Creating RefSeq table")
    conn = sqlite3.connect(database)
    cursor = conn.cursor()
    cursor.execute("DROP TABLE IF EXISTS refseq")
    cursor.execute('CREATE TABLE refseq \
        (gene_id TEXT, product TEXT, taxonomy TEXT, \
        ec_number TEXT, ko_number TEXT, ko_product TEXT)')
    with open(table, 'r') as refseq:
        record_counter = 0
        records = []
        for line in refseq:
            # Commit changes after 500000 records
            if record_counter == 500000:
                cursor.execute("begin")
                cursor.executemany('INSERT INTO refseq VALUES(?, ?, ?, ?, ?, ?)', records)
                cursor.execute("commit")
                record_counter = 0
                records = []
            else:
                line = line.rstrip('\n').split("\t")
                if line[0] in protein_annotations:
                    ko_number = protein_annotations[line[0]][0]
                    ko_product = protein_annotations[line[0]][1]
                    records.append((line[0], line[1], line[2], line[3], ko_number, ko_product))
                    record_counter += 1
                else:
                    records.append((line[0], line[1], line[2], line[3], "NA", "NA"))
                    record_counter += 1
        # Commit remaining records
        if record_counter > 0:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO refseq VALUES(?, ?, ?, ?, ?, ?)', records)
            cursor.execute("commit")
        # Create index for faster access
        cursor.execute('CREATE INDEX refseq_id ON refseq (gene_id)')
    print("Done")


################################################################################
"""---2.0 Main Function---"""

def main():
    import argparse, sys
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''This script builds a sqlite database from a tab separated table.\n'''
            '''By default it assumes the first line of the input table has headers, if not\n'''
            '''you must provide a list of headers as --header header1,header2,header3...\n'''
            '''Usage: ''' + sys.argv[0] + ''' -i [Input Table] -d [Database Name]\n'''
            '''Global mandatory parameters: -i [Input Table] -d [Database Name]\n'''
            '''Optional Database Parameters: See ''' + sys.argv[0] + ' -h')
    parser.add_argument('-i', '--input', dest='input_table', action='store', required=True,
                        help='Input tab-delimited table to parse, by default assumes headers are present')
    parser.add_argument('-d', '--database', dest='database', action='store', required=True,
                        help='Database name, can be exisiting or new.')
    parser.add_argument('--type', dest='db_type', action='store', required=True,
                        help='Table type, can be one of swissprot, trembl, or refseq')
    args = parser.parse_args()

    input_table = args.input_table
    database = args.database
    db_type = args.db_type
    db_type = db_type.lower()

    # ----------------------------
    print("Building database")
    if db_type == "swissprot":
        create_swissprot_table(database, input_table)
    elif db_type == "trembl":
        create_trembl_table(database, input_table)
    elif db_type == "refseq":
        create_refseq_table(database, input_table)
    else:
        exit("Please specify a table type, swissprot, trembl or refseq")
    print("Database succesfully created!")
    # ----------------------------

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: Finds identifiers from one database into another.
# Uses the interconversion tables in the main database.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
import sqlite3

################################################################################
"""---1.0 Define Functions---"""
def convert_refseq_to_uniprot(identifier_list, sql_database, inverse):
    conn = sqlite3.connect(sql_database)
    cursor = conn.cursor()
    refseq_to_uniprot = {}
    uniprot_to_refseq = {}
    for identifier in identifier_list:
        # If uniprot to refseq
        if inverse == False:
            cursor.execute("SELECT * FROM refseq_to_uniprot WHERE uniprot_id=?", (identifier,))
            rows = cursor.fetchall()
            if len(rows) > 0:
                uniprot_to_refseq[identifier] = []
                for match in rows:
                    # Append refseq matches
                    uniprot_to_refseq[identifier].append(match[0])
            return uniprot_to_refseq
        # Else search refseq to uniprot
        else:
            cursor.execute("SELECT * FROM refseq_to_uniprot WHERE refseq_id=?", (identifier,))
            rows = cursor.fetchall()
            if len(rows) > 0:
                refseq_to_uniprot[identifier] = []
                for match in rows:
                    # Append uniprot matches
                    refseq_to_uniprot[identifier].append(match[1])
            return refseq_to_uniprot


def convert_ko_to_ec(identifier_list, sql_database, inverse):
    conn = sqlite3.connect(sql_database)
    cursor = conn.cursor()
    ko_to_ec = {}
    ec_to_ko = {}
    for identifier in identifier_list:
        # If ec to ko
        if inverse == False:
            cursor.execute("SELECT * FROM ko_to_ec WHERE ec_identifier=?", (identifier,))
            rows = cursor.fetchall()
            if len(rows) > 0:
                ec_to_ko[identifier] = []
                for match in rows:
                    # Append ko matches
                    ec_to_ko[identifier].append(match[0])
            return ec_to_ko
        # Else search ko to ec
        else:
            cursor.execute("SELECT * FROM ko_to_ec WHERE ko_identifier=?", (identifier,))
            rows = cursor.fetchall()
            if len(rows) > 0:
                ko_to_ec[identifier] = []
                for match in rows:
                    # Append ec matches
                    ko_to_ec[identifier].append(match[1])
            return ko_to_ec

def convert_interpro_to_ko(identifier_list, sql_database, inverse):
    conn = sqlite3.connect(sql_database)
    cursor = conn.cursor()
    interpro_to_ko = {}
    ko_to_interpro = {}
    for identifier in identifier_list:
        # If ko to interpro
        if inverse == True:
            cursor.execute("SELECT * FROM interpro_to_ko WHERE ko_identifier=?", (identifier,))
            rows = cursor.fetchall()
            if len(rows) > 0:
                ko_to_interpro[identifier] = []
                for match in rows:
                    # Append ko matches
                    ko_to_interpro[identifier].append(match[0])
            return ko_to_interpro
        # Else search interpro to ko
        else:
            cursor.execute("SELECT * FROM interpro_to_ko WHERE interpro_id=?", (identifier,))
            rows = cursor.fetchall()
            if len(rows) > 0:
                interpro_to_ko[identifier] = []
                for match in rows:
                    # Append ec matches
                    interpro_to_ko[identifier].append(match[1])
            return interpro_to_ko

def convert_interpro_to_ec(identifier_list, sql_database, inverse):
    conn = sqlite3.connect(sql_database)
    cursor = conn.cursor()
    interpro_to_ec = {}
    ec_to_interpro = {}
    for identifier in identifier_list:
        # If ko to interpro
        if inverse == True:
            cursor.execute("SELECT * FROM interpro_to_ec WHERE ec_identifier=?", (identifier,))
            rows = cursor.fetchall()
            if len(rows) > 0:
                ec_to_interpro[identifier] = []
                for match in rows:
                    # Append ko matches
                    ec_to_interpro[identifier].append(match[0])
            return ec_to_interpro
        # Else search interpro to ko
        else:
            cursor.execute("SELECT * FROM interpro_to_ec WHERE interpro_id=?", (identifier,))
            rows = cursor.fetchall()
            if len(rows) > 0:
                interpro_to_ec[identifier] = []
                for match in rows:
                    # Append ec matches
                    interpro_to_ec[identifier].append(match[1])
            return interpro_to_ec


################################################################################
"""---2.0 Main Function---"""

def main():
    import argparse, sys
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''This script searches for identifiers and returns associated ids from other databases.\n'''
            '''Usage: ''' + sys.argv[0] + ''' -f [output_file folder]\n'''
            '''Global mandatory parameters: -f [output_file folder]\n'''
            '''Optional Database Parameters: See ''' + sys.argv[0] + ' -h')
    general_options = parser.add_argument_group('Inputs and outputs.')
    general_options.add_argument('-i', '--input_file', dest='input_file', action='store', required=True,
                        help='File with identifiers to search and convert (one per line).')
    general_options.add_argument('-d', '--database', dest='database', action='store', required=True,
                        help='SQLite database to search.')
    general_options.add_argument('-o', '--outfile', dest='outfile', action='store', required=True,
                        help='Output table with original id and corresponding ids in other databases.')
    conversion_to_perform = parser.add_argument_group('Available conversions.')
    conversion_to_perform.add_argument('--refseq_to_uniprot', dest='refseq_to_uniprot', action='store_true', required=False,
                        help='Convert refseq -> uniprot or uniprot -> refseq (see --inverse)')
    conversion_to_perform.add_argument('--ko_to_ec', dest='ko_to_ec', action='store_true', required=False,
                        help='Convert ko -> ec or ec -> ko (see --inverse)')
    conversion_to_perform.add_argument('--interpro_to_ko', dest='interpro_to_ko', action='store_true', required=False,
                        help='Convert interpro -> ko or ko -> interpro (see --inverse)')
    conversion_to_perform.add_argument('--interpro_to_ec', dest='interpro_to_ec', action='store_true', required=False,
                        help='Convert interpro -> ec or ec -> interpro (see --inverse)')
    conversion_modificatopn = parser.add_argument_group('Changes to searching order.')
    conversion_modificatopn.add_argument('--inverse', dest='inverse', action='store_true', required=False, default=False,
                        help='Change the order of conversion. For example, in ko_to_ec instead to converting ko -> ec, convert ec -> ko.')
    args = parser.parse_args()

    input_file = args.input_file
    database = args.database
    outfile = args.outfile
    refseq_to_uniprot = args.refseq_to_uniprot
    ko_to_ec = args.ko_to_ec
    interpro_to_ko = args.interpro_to_ko
    interpro_to_ec = args.interpro_to_ec
    inverse = args.inverse

    # ----------------------------
    print("Checking inputs...")
    # Check if more than one or no conversion was selected
    conversion_types = [refseq_to_uniprot, ko_to_ec, interpro_to_ko, interpro_to_ec]
    if conversion_types.count(True) == 0:
        exit("Plase select one type of conversion to perform.")
    elif conversion_types.count(True) > 1:
        exit("Plase select only one type of conversion to perform.")
    
    # Parse the input file and get the identifiers
    identifier_list = []
    with open(input_file, 'r') as infile:
        for line in infile:
            identifier_list.append(line.strip().split()[0])
    print("Done!")

    print("Searching in the database...")
    corresponding_ids = None
    if refseq_to_uniprot == True:
        corresponding_ids = convert_refseq_to_uniprot(identifier_list, database, inverse)
    elif ko_to_ec == True:
        corresponding_ids = convert_ko_to_ec(identifier_list, database, inverse)
    elif interpro_to_ko == True:
        corresponding_ids = convert_interpro_to_ko(identifier_list, database, inverse)
    elif interpro_to_ec == True:
        corresponding_ids = convert_interpro_to_ec(identifier_list, database, inverse)
    print("Done!")

    print("Writing output...")
    with open(outfile, 'w') as output:
        for identifier, matches in corresponding_ids.items():
            for match in matches:
                output.write("{}\t{}\n".format(identifier, match))
    print("Done!")
    # ----------------------------

if __name__ == "__main__":
    main()


#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: Builds the search databases depending on the search method
# selected. Can be one of blast, sword or diamond. These must be in the PATH
# or the path must be provided.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
import subprocess
from shutil import which
from sys import exit
from pathlib import Path

################################################################################
"""---1.0 Define Functions---"""
def blastp_db_creator(directory, path=None):
    print('Building blast databases')
    database_files = []
    if path != None:
        makeblasdb_call = Path(path) / 'makeblastdb'
    else:
        makeblasdb_call = 'makeblastdb'
    if which(makeblasdb_call) == None:
        script_path = Path(__file__)
        script_dir = script_path.parent
        stand_alone_script = script_dir / "protein_db_creation.py"
        print('Blast\'s "makeblastdb" not found in path, I cannot continue and so, I will exit.')
        print('If you are running the microbeannotator_db_builder script this is the last step of the process.')
        print('Given that this last step failed you need to run the standalone ' + str(stand_alone_script) + ' script')
        print('and provide the binary folder for the selected method using the "--bin_path" option or add')
        print('the binary to your PATH (see README for help on this).')
        exit('Otherwise, just re-run this script providing the appropriate path. :)')
    else:
        for protein_file in Path(directory).iterdir():
            if protein_file.suffix == '.fasta':
                output_file_name = protein_file.with_suffix("")
                database_files.append(output_file_name)
                subprocess.call([makeblasdb_call, '-in', protein_file, '-dbtype', 'prot',
                '-out', output_file_name])
    print('Done!')
    with open(Path(directory)/'blast_db.list', 'w') as db_list:
        for file in database_files:
            db_list.write('{}\n'.format(file.name))

def diamond_db_creator(directory, threads, path=None):
    print('Building diamond databases')
    database_files = []
    if path != None:
        diamond_call = Path(path) / 'diamond'
    else:
        diamond_call = 'diamond'
    if which(diamond_call) == None:
        script_path = Path(__file__)
        script_dir = script_path.parent
        stand_alone_script = script_dir / "protein_db_creation.py"
        print('Diamond\'s "makedb" not found in path, I cannot continue and so, I will exit.')
        print('If you are running the microbeannotator_db_builder script this is the last step of the process.')
        print('Given that this last step failed you need to run the standalone ' + str(stand_alone_script) + ' script')
        print('and provide the binary folder for the selected method using the "--bin_path" option or add')
        print('the binary to your PATH (see README for help on this).')
        exit('Otherwise, just re-run this script providing the appropriate path. :)')
    else:
        for protein_file in Path(directory).iterdir():
            if protein_file.suffix == '.fasta':
                output_file_name = protein_file.with_suffix("")
                final_db_name = Path(output_file_name).with_suffix('.dmnd')
                database_files.append(final_db_name)
                subprocess.call([diamond_call, 'makedb', '--in', protein_file, '-d', output_file_name, '--threads', str(threads)])
    print('Done!')
    with open(Path(directory)/'diamond_db.list', 'w') as db_list:
        for file in database_files:
            db_list.write('{}\n'.format(file.name))

def sword_db_creator(directory, path=None):
    print('Building sword databases (no db needed, just checking if sword is in path)')
    if path != None:
        sword_call = Path(path) / 'sword'
    else:
        sword_call = 'sword'
    if which(sword_call) == None:
        print('I did not find the sword executable. I don\'t need')
        print('it for this step. However, the search might fail in the future.')
        print('Make sure to provide the correct path for sword when searching.')
    else:
        print('Done!')
    with open(Path(directory)/'sword_db.list', 'w') as db_list:
        for file in Path(directory).iterdir():
            if file.suffix == '.fasta':
                db_list.write('{}\n'.format(file.name))


################################################################################
"""---2.0 Main Function---"""

def main():
    import argparse, sys
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''This script downloads the fasta and genbank files that\n'''
            '''are needed to build the RefSeq annotation database. By default it\n'''
            '''downloads both but you can specify either\n'''
            '''Usage: ''' + sys.argv[0] + ''' -f [output_file folder]\n'''
            '''Global mandatory parameters: -f [output_file folder]\n'''
            '''Optional Database Parameters: See ''' + sys.argv[0] + ' -h')
    parser.add_argument('-f', '--folder', dest='folder', action='store', required=True,
                        help='Folder where all raw fasta files are located.')
    parser.add_argument('-m', '--method', dest='method', action='store', required=True,
                        help='Search (and db creation) method, one of blast, diamond or sword')
    parser.add_argument('--bin_path', dest='bin_path', action='store', required=False,
                        help='Path to binary folder for selected method.')
    args = parser.parse_args()

    folder = args.folder
    method = args.method
    bin_path = args.bin_path
    method = method.lower()

    # ----------------------------
    if method == 'blast':
        blastp_db_creator(folder, bin_path)
    elif method == 'diamond':
        diamond_db_creator(folder, bin_path)
    elif method == 'sword':
        sword_db_creator(folder, bin_path)
    # ----------------------------

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: Parses compressed genbank files and extracts protein
information relevant for annotation purposes.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
import argparse
from sys import argv
import gzip
import multiprocessing
import shutil
from pathlib import Path
from Bio import SeqIO

################################################################################
"""---1.0 Define Functions---"""

def table_creator(genbank_file):
    """ Parses genbank files and extract annotation information
    
    Arguments:
        genbank_file {str} -- Path to compressed genbank file
    
    Returns:
        temporal_table -- Path to temporal table created
    """
    temporal_table = Path(genbank_file).with_suffix('.temp')
    with gzip.open(genbank_file, 'rt') as uncompressed_genbank, \
    open(temporal_table, 'w') as output_file:
        for record in SeqIO.parse(uncompressed_genbank, "genbank"):
            protein_id = record.id
            product = record.description.split("[")[0].strip()
            taxonomy = ""
            species = ""
            ec_number = ""
            if 'organism' not in record.annotations:
                species = 'NA'
            else:
                species = record.annotations['organism']
                if 'taxonomy' not in record.annotations and species == "NA":
                    taxonomy = 'NA'
                elif 'taxonomy' not in record.annotations and species != "NA":
                    taxonomy = species
                else:
                    taxonomy = ", ".join(record.annotations['taxonomy']) + ", " + species
                if product == "" or "unknown" in product:
                    for feature in record.features:
                        if feature.type == "Protein":
                            if 'product' not in feature.qualifiers:
                                if 'name' in feature.qualifiers:
                                    product = feature.qualifiers['name'][0]
                                else:
                                    product = 'NA'
                            else:
                                product = feature.qualifiers['product'][0]
            for feature in record.features:
                if feature.type == "Protein" and "EC_number" in feature.qualifiers:
                    ec_number = feature.qualifiers["EC_number"][0]
                else:
                    ec_number = "NA"
            output_file.write("{}\t{}\t{}\t{}\n".format(protein_id, product, taxonomy, ec_number))
    return temporal_table


def table_generator_worker(genbank_list, threads):
    """ Worker calling table_creator function
    
    Arguments:
        genbank_list {list} -- List of compressed GenBank files
        threads {int} -- Threads to use
    
    Returns:
        temp_table_list -- List of temporal tables created
    """
    temp_table_list = []
    try:
        pool = multiprocessing.Pool(threads)
        temp_table_list = pool.map(table_creator, genbank_list)
    finally:
        pool.close()
        pool.join()
    return temp_table_list


def table_merger(temp_table_list, final_table, keep):
    """ Merges individual tables into a refseq annotation table
    
    Arguments:
        temp_table_list {list} -- List of paths to temporal tables created
        final_table {str} -- Final output_file table
        keep {bool} -- Retain intermediate files
    """
    with open(final_table, 'w') as output_file:
        for file in temp_table_list:
            with open(file) as temp:
                shutil.copyfileobj(temp, output_file)
            if keep == False:
                file.unlink()
            else: continue


################################################################################
"""---3.0 Main Function---"""
def main():
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''This script parses the downloaded compressed RefSeq Genbank files,\n'''
                        '''extracts relevant information and stores in a single table\n'''
            '''Usage: ''' + argv[0] + ''' -i [Genbank Files] -t [Threads] -o [output_file]\n'''
            '''Global mandatory parameters: -i [Genbank Files] -o [output_file]\n'''
            '''Optional Database Parameters: See ''' + argv[0] + ' -h')

    input_options = parser.add_argument_group('General input/output_file options')
    input_options.add_argument('-i', '--input_files', dest='input_files', action='store', nargs='+', required=True,
                        help='Space-separated list of compressed genbank files, or use something like "$(ls files*.gz)"')
    input_options.add_argument('-o', '--output_file', dest='output_file', action='store', required=True,
                        help='File to store final table with record information')

    misc_options = parser.add_argument_group('Miscellaneous options')
    misc_options.add_argument('-t', '--threads', dest='threads', action='store', default=1, type=int, required=False,
                        help='Number of threads to use, by default 1')
    misc_options.add_argument('-k', '--keep', dest='keep', action='store_true', required=False,
                        help='Keep intermediate files, by default false')

    args = parser.parse_args()
    
    input_files = args.input_files
    output_file = args.output_file
    threads = args.threads
    keep = args.keep

    # --------------------------------------------------
    print("Processing {} files on {} threads...".format(len(input_files), threads), end=" ")
    temp_tables = table_generator_worker(input_files, threads)
    print("Done")
    print("Merging tables...", end=" ")
    table_merger(temp_tables, output_file, keep)
    print("Done")
    # --------------------------------------------------


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: Downloads the information necessary to create interconversion of
# accessions and appends it into the MicrobeAnnotator database.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
import wget
from pathlib import Path
import gzip
import sqlite3

################################################################################
"""---1.0 Define Functions---"""
def create_refseq_to_uniprot(input_table, database_to_append, keep):
    # Connect to database
    conn = sqlite3.connect(database_to_append)
    cursor = conn.cursor()
    cursor.execute("DROP TABLE IF EXISTS refseq_to_uniprot")
    cursor.execute('CREATE TABLE refseq_to_uniprot \
        (refseq_id TEXT, uniprot_id TEXT)')
    # Read input table and create table within SQLite database
    with open(input_table, 'r') as refseq_conversion:
        record_counter = 0
        records = []
        for line in refseq_conversion:
            # Commit changes after 500000 records
            if record_counter == 500000:
                cursor.execute("begin")
                cursor.executemany('INSERT INTO refseq_to_uniprot VALUES(?, ?)', records)
                cursor.execute("commit")
                record_counter = 0
                records = []
            else:
                line = line.strip('\n').split("\t")
                records.append((line[1], line[0]))
                record_counter += 1
        # Commit remaining records
        if record_counter > 0:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO refseq_to_uniprot VALUES(?, ?)', records)
            cursor.execute("commit")
        # Create index for faster access
        cursor.execute('CREATE INDEX refseq_index ON refseq_to_uniprot (refseq_id)')
    # Remove temporal files
    if keep == False:
        if Path(input_table).is_file():
            Path(input_table).unlink()

def create_ko_to_ec(output_folder, database, keep):
    # Download information
    download_output = str(output_folder / "02.KO_to_EC_identifiers.txt.gz")
    wget.download("http://enve-omics.ce.gatech.edu/data/public_microbeannotator/02.KO_to_EC_identifiers.txt.gz",
                            out=download_output)
    # Connect with database
    conn = sqlite3.connect(database)
    cursor = conn.cursor()
    # Create table with correspondence KO -> EC
    cursor.execute("DROP TABLE IF EXISTS ko_to_ec")
    cursor.execute('CREATE TABLE ko_to_ec \
        (ko_identifier TEXT, ec_identifier TEXT)')
    with gzip.open(download_output, 'rt') as ko_conversion:
        record_counter = 0
        records = []
        for line in ko_conversion:
            # Commit changes after 500000 records
            if record_counter == 500000:
                cursor.execute("begin")
                cursor.executemany('INSERT INTO ko_to_ec VALUES(?, ?)', records)
                cursor.execute("commit")
                record_counter = 0
                records = []
            else:
                line = line.strip('\n').split("\t")
                records.append((line[0], line[1]))
                record_counter += 1
        # Commit remaining records
        if record_counter > 0:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO ko_to_ec VALUES(?, ?)', records)
            cursor.execute("commit")
        # Create index for faster access
    cursor.execute('CREATE INDEX ko_id ON ko_to_ec (ko_identifier)')
    cursor.execute('CREATE INDEX ec_id ON ko_to_ec (ec_identifier)')

    # Remove temporal files
    if keep == False:
        if Path(download_output).is_file():
            Path(download_output).unlink()
    
def create_interpro_tables(output_folder, database, keep):
    # Download information
    download_output = str(output_folder / "interpro_metadata.xml.gz")
    wget.download("ftp://ftp.ebi.ac.uk/pub/databases/interpro/interpro.xml.gz",
                            out=download_output)
    interpro_to_ec = {}
    interpro_to_ko = {}
    ec_to_interpro = {}
    ko_to_interpro = {}
    # Parse file information into dictionaries to merge into the SQLite database
    with gzip.open(download_output, 'rt') as infile:
        interpro_id = ""
        ec_identifier = []
        ko_identifier = []
        for line in infile:
            if line.startswith("<interpro id="):
                interpro_id = line.strip().split('"')[1]
            elif "db_xref" in line and "EC" in line:
                identifier = line.strip().split('"')[3]
                ec_identifier.append(identifier)
                if ec_identifier in ec_to_interpro:
                    ec_to_interpro[ec_identifier].append(interpro_id)
                else:
                    ec_to_interpro[ec_identifier] = [interpro_id]
            elif "db_xref" in line and "KEGG" in line:
                identifier = line.strip().split('"')[3]
                ko_identifier.append(identifier)
                if ec_identifier in ko_to_interpro:
                    ko_to_interpro[ec_identifier].append(interpro_id)
                else:
                    ko_to_interpro[ec_identifier] = [interpro_id]
            elif line.startswith("</interpro>"):
                interpro_to_ec[interpro_id] = ec_identifier
                interpro_to_ko[interpro_id] = ko_identifier
                interpro_id = ""
                ec_identifier = []
                ko_identifier = []
    # Connect with the database
    conn = sqlite3.connect(database)
    cursor = conn.cursor()
    # Create table with correspondence InterPro -> KO
    cursor.execute("DROP TABLE IF EXISTS interpro_to_ko")
    cursor.execute('CREATE TABLE interpro_to_ko \
        (interpro_id TEXT, ko_identifier TEXT)')
    record_counter = 0
    records = []
    for interproscan, ko_id in interpro_to_ko.items():
        # Commit changes after 500000 records
        if record_counter == 500000:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO interpro_to_ko VALUES(?, ?)', records)
            cursor.execute("commit")
            record_counter = 0
            records = []
        else:
            records.append((interproscan, " ".join(ko_id)))
            record_counter += 1
        # Commit remaining records
        if record_counter > 0:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO interpro_to_ko VALUES(?, ?)', records)
            cursor.execute("commit")
        # Create index for faster access
    cursor.execute('CREATE INDEX interpro_index_ko ON interpro_to_ko (interpro_id)')

    # Create table with correspondence KO -> Interpro
    cursor.execute("DROP TABLE IF EXISTS ko_to_interpro")
    cursor.execute('CREATE TABLE ko_to_interpro \
        (ko_identifier TEXT, interpro_id TEXT)')
    record_counter = 0
    records = []
    for ko_id, interproscan in ko_to_interpro.items():
        # Commit changes after 500000 records
        if record_counter == 500000:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO ko_to_interpro VALUES(?, ?)', records)
            cursor.execute("commit")
            record_counter = 0
            records = []
        else:
            records.append((ko_id, " ".join(interproscan)))
            record_counter += 1
        # Commit remaining records
        if record_counter > 0:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO ko_to_interpro VALUES(?, ?)', records)
            cursor.execute("commit")
        # Create index for faster access
    cursor.execute('CREATE INDEX ko_index_interpro ON ko_to_interpro (ko_identifier)')

    # Create table with correspondence InterPro -> EC
    cursor.execute("DROP TABLE IF EXISTS interpro_to_ec")
    cursor.execute('CREATE TABLE interpro_to_ec \
        (interpro_id TEXT, ec_identifier TEXT)')
    record_counter = 0
    records = []
    for interproscan, ec_id in interpro_to_ec.items():
        # Commit changes after 500000 records
        if record_counter == 500000:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO interpro_to_ec VALUES(?, ?)', records)
            cursor.execute("commit")
            record_counter = 0
            records = []
        else:
            records.append((interproscan, " ".join(ec_id)))
            record_counter += 1
        # Commit remaining records
        if record_counter > 0:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO interpro_to_ec VALUES(?, ?)', records)
            cursor.execute("commit")
        # Create index for faster access
    cursor.execute('CREATE INDEX interpro_index_ce ON interpro_to_ec (interpro_id)')

    # Create table with correspondence EC -> Interpro
    cursor.execute("DROP TABLE IF EXISTS ec_to_interpro")
    cursor.execute('CREATE TABLE ec_to_interpro \
        (ec_identifier TEXT, interpro_id TEXT)')
    record_counter = 0
    records = []
    for ec_id, interproscan in ec_to_interpro.items():
        # Commit changes after 500000 records
        if record_counter == 500000:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO ec_to_interpro VALUES(?, ?)', records)
            cursor.execute("commit")
            record_counter = 0
            records = []
        else:
            records.append((ec_id, " ".join(interproscan)))
            record_counter += 1
        # Commit remaining records
        if record_counter > 0:
            cursor.execute("begin")
            cursor.executemany('INSERT INTO ec_to_interpro VALUES(?, ?)', records)
            cursor.execute("commit")
        # Create index for faster access
    cursor.execute('CREATE INDEX ce_index_interpro ON ec_to_interpro (ec_identifier)')

    # Remove temporal files
    if keep == False:
        if Path(download_output).is_file():
            Path(download_output).unlink()

################################################################################
"""---2.0 Main Function---"""

def main():
    import argparse, sys
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''This script build the search databases required by MicrobeAnnotator\n'''
            '''Usage: ''' + sys.argv[0] + ''' -f [output_file folder]\n'''
            '''Global mandatory parameters: -f [output_file folder]\n'''
            '''Optional Database Parameters: See ''' + sys.argv[0] + ' -h')
    parser.add_argument('-i', '--input_table', dest='intable', action='store', required=True,
                        help='RefSeq to Uniprot table to create correspondence table.')
    parser.add_argument('-d', '--database', dest='database', action='store', required=True,
                        help='SQLite database to create tables in.')
    parser.add_argument('-o', '--output', dest='output', action='store', required=True,
                        help='Output directory where data should be stored.')
    parser.add_argument('--keep', dest='keep', action='store_true', required=False,
                        help='Keep intermediate files, can increase disk requirement (not necessary and therefore not recommended).')
    args = parser.parse_args()

    intable = args.intable
    database = args.database
    output = args.output
    keep = args.keep

    # ----------------------------
    print("Creating correspondence tables...")
    try:
        create_refseq_to_uniprot(intable, database, keep)
    except:
        print("Could not create RefSeq to Uniprot table.")
    try:
        create_ko_to_ec(output, database, keep)
    except:
        print("Could not create KO to EC table.")
    try:
        create_interpro_tables(output, database, keep)
    except:
        print("Could not create UniProt tables.")
    print("Done!")
    # ----------------------------

if __name__ == "__main__":
    main()




#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: Download the latest refseq protein dataset from NCBI and
# the accompanying genbank files necessary to create the database.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
from shutil import which
from shutil import copyfileobj
from shutil import rmtree
import subprocess
from pathlib import Path
import urllib.request
import gzip
import wget
import multiprocessing

################################################################################
"""---1.0 Define Functions---"""
def refseq_fasta_downloader(output_file_folder, ascp_key = None):
    print("Downloading protein fasta files")
    # Requirements
    if ascp_key == None:
        ascp_key = get_aspera_key()
    release_file = urllib.request.urlopen("https://ftp.ncbi.nlm.nih.gov/refseq/release/RELEASE_NUMBER")
    release_number = release_file.readline().decode('utf-8').strip()
    # Create folders
    temp_fasta_files = Path(output_file_folder) / "01.temp_proteins"
    merged_db_folder = Path(output_file_folder) / "01.Protein_DB"
    Path(temp_fasta_files).mkdir(parents=True, exist_ok=True)
    Path(merged_db_folder).mkdir(parents=True, exist_ok=True)
    with open(merged_db_folder / "refseq_release.txt", 'w') as relinfo:
        relinfo.write("Release number {}".format(release_number))
    # Remove refseq fasta files if present (to avoid repeating files)
    if Path(merged_db_folder / "refseq_protein.fasta").is_file():
        Path(merged_db_folder / "refseq_protein.fasta").unlink()
    # Download compressed protein files
    subprocess.call(["ascp", '-QTr', '-d', '-l', '100M', '-E', '*wgs_mstr*',
                    '-E', '*rna*', '-E', '*genomic*', '-E', '*.gpff.gz',
                    '-i', ascp_key, 'anonftp@ftp.ncbi.nlm.nih.gov:/refseq/release/viral/',
                    temp_fasta_files])
    subprocess.call(["ascp", '-QTr', '-d', '-l', '100M', '-E', '*wgs_mstr*',
                    '-E', '*rna*', '-E', '*genomic*', '-E', '*.gpff.gz',
                    '-i', ascp_key, 'anonftp@ftp.ncbi.nlm.nih.gov:/refseq/release/bacteria/',
                    temp_fasta_files])
    subprocess.call(["ascp", '-QTr', '-d', '-l', '100M', '-E', '*wgs_mstr*',
                    '-E', '*rna*', '-E', '*genomic*', '-E', '*.gpff.gz',
                    '-i', ascp_key, 'anonftp@ftp.ncbi.nlm.nih.gov:/refseq/release/archaea/',
                    temp_fasta_files])
    # Get downloaded files
    viral_list = search_all_files(temp_fasta_files / "viral")
    bacteria_list = search_all_files(temp_fasta_files / "bacteria")
    archaea_list = search_all_files(temp_fasta_files / "archaea")
    final_list = viral_list + bacteria_list + archaea_list
    with open(Path(merged_db_folder) / ("refseq_protein.fasta"), 'w') as merged_db:
        for file in final_list:
            with gzip.open(file, 'rt') as temp_file:
                copyfileobj(temp_file,merged_db)
                Path.unlink(file)
    rmtree(temp_fasta_files)
    
def refseq_fasta_downloader_wget(output_file_folder, threads):
    print("Downloading protein fasta files")
    # Extract database release number 
    release_file = urllib.request.urlopen("https://ftp.ncbi.nlm.nih.gov/refseq/release/RELEASE_NUMBER")
    release_number = release_file.readline().decode('utf-8').strip()
    # Create folders
    temp_fasta_files = Path(output_file_folder) / "01.temp_proteins"
    merged_db_folder = Path(output_file_folder) / "01.Protein_DB"
    Path(temp_fasta_files).mkdir(parents=True, exist_ok=True)
    Path(merged_db_folder).mkdir(parents=True, exist_ok=True)
    with open(merged_db_folder / "refseq_release.txt", 'w') as relinfo:
        relinfo.write("Release number {}".format(release_number))
    # Remove refseq fasta files if present (to avoid repeating files)
    if Path(merged_db_folder / "refseq_protein.fasta").is_file():
        Path(merged_db_folder / "refseq_protein.fasta").unlink()
    # Download compressed protein files using wget
    file_list = []
    for domain in ["viral", "bacteria", "archaea"]:
        ncbi_url = "https://ftp.ncbi.nlm.nih.gov/refseq/release/" + domain
        domain_url_info = urllib.request.urlopen(ncbi_url)
        for line in domain_url_info:
            line = line.decode('utf-8').strip()
            if "protein.faa.gz" in line:
                filename = line.split('"')[1]
                file_url = "ftp://ftp.ncbi.nlm.nih.gov/refseq/release/" + domain + "/" + str(filename)
                output = str(temp_fasta_files) + "/" + str(filename)
                file_list.append((file_url, output))
    try:
        pool = multiprocessing.Pool(threads)
        pool.map(refseq_multiprocess_downloader, file_list)
    finally:
        pool.close()
        pool.join()
    # Get downloaded files
    final_list = search_all_files(temp_fasta_files)
    with open(Path(merged_db_folder) / ("refseq_protein.fasta"), 'w') as merged_db:
        for file in final_list:
            with gzip.open(file, 'rt') as temp_file:
                copyfileobj(temp_file,merged_db)
                Path.unlink(file)
    rmtree(temp_fasta_files)


def refseq_genbank_downloader(output_file_folder, ascp_key = None):
    print("Downloading protein genbank files")
    # Requirements
    if ascp_key == None:
        ascp_key = get_aspera_key()
    temp_gb_files = Path(output_file_folder) / "02.temp_genbank"
    Path(temp_gb_files).mkdir(parents=True, exist_ok=True)
    # Download compressed protein files
    subprocess.call(["ascp", '-QTr', '-d', '-l', '100M', '-E', '*wgs_mstr*',
                    '-E', '*rna*', '-E', '*genomic*', '-E', '*.faa.gz',
                    '-i', ascp_key, 'anonftp@ftp.ncbi.nlm.nih.gov:/refseq/release/viral/',
                    temp_gb_files])
    subprocess.call(["ascp", '-QTr', '-d', '-l', '100M', '-E', '*wgs_mstr*',
                    '-E', '*rna*', '-E', '*genomic*', '-E', '*.faa.gz',
                    '-i', ascp_key, 'anonftp@ftp.ncbi.nlm.nih.gov:/refseq/release/bacteria/',
                    temp_gb_files])
    subprocess.call(["ascp", '-QTr', '-d', '-l', '100M', '-E', '*wgs_mstr*',
                    '-E', '*rna*', '-E', '*genomic*', '-E', '*.faa.gz',
                    '-i', ascp_key, 'anonftp@ftp.ncbi.nlm.nih.gov:/refseq/release/archaea/',
                    temp_gb_files])
    # Get downloaded files
    viral_list = search_all_files(temp_gb_files / "viral")
    bacteria_list = search_all_files(temp_gb_files / "bacteria")
    archaea_list = search_all_files(temp_gb_files / "archaea")
    final_list = viral_list + bacteria_list + archaea_list
    return final_list

def refseq_genbank_downloader_wget(output_file_folder, threads):
    print("Downloading protein genbank files")
    # Requirements
    temp_gb_files = Path(output_file_folder) / "02.temp_genbank"
    Path(temp_gb_files).mkdir(parents=True, exist_ok=True)
    # Download compressed genbank files using wget
    file_list = []
    for domain in ["viral", "bacteria", "archaea"]:
        ncbi_url = "https://ftp.ncbi.nlm.nih.gov/refseq/release/" + domain
        domain_url_info = urllib.request.urlopen(ncbi_url)
        for line in domain_url_info:
            line = line.decode('utf-8').strip()
            if "protein.gpff.gz" in line:
                filename = line.split('"')[1]
                file_url = "ftp://ftp.ncbi.nlm.nih.gov/refseq/release/" + domain + "/" + str(filename)
                output = str(temp_gb_files) + "/" + str(filename)
                file_list.append((file_url, output))
    # Download using multiple cores
    try:
        pool = multiprocessing.Pool(threads)
        pool.map(refseq_multiprocess_downloader, file_list)
    finally:
        pool.close()
        pool.join()
    final_list = search_all_files(temp_gb_files)
    return final_list

def refseq_multiprocess_downloader(file_list):
    file_url = file_list[0]
    output = file_list[1]
    wget.download(file_url, out=output)

def get_aspera_key():
    path_to_ascp = which("ascp")
    install_folder = Path(path_to_ascp).parents[1]
    key = str(install_folder) + "/etc/asperaweb_id_dsa.openssh"
    return key

def search_all_files(directory):
    dirpath = Path(directory)
    assert(dirpath.is_dir())
    file_list = []
    for file in dirpath.iterdir():
        if file.is_file() and file.suffix == ".gz":
            file_list.append(file)
        else:
            continue
    return file_list


################################################################################
"""---2.0 Main Function---"""

def main():
    import argparse, sys
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''This script downloads the fasta and genbank files that\n'''
            '''are needed to build the RefSeq annotation database. By default it\n'''
            '''downloads both but you can specify either.\n'''
            '''Usage: ''' + sys.argv[0] + ''' -f [output_file folder]\n'''
            '''Global mandatory parameters: -f [output_file folder]\n'''
            '''Optional Database Parameters: See ''' + sys.argv[0] + ' -h')
    parser.add_argument('-f', '--folder', dest='folder', action='store', required=False,
                        help='Folder to store the fasta and genbank files')
    parser.add_argument('--proteins', dest='proteins', action='store_true', required=False,
                        help='Only download the protein fasta files.')
    parser.add_argument('--genbank', dest='genbank', action='store_true', required=False,
                        help='Only download the genbank files')
    parser.add_argument('--ascp', dest='ascp_key', action='store', required=False, default=None,
                        help='Path to Aspera Connect key')
    args = parser.parse_args()

    folder = args.folder
    proteins = args.proteins
    genbank = args.genbank
    ascp_key = args.ascp_key

    # ----------------------------
    if proteins == False and genbank == False:
        refseq_fasta_downloader(folder, ascp_key)
        refseq_genbank_downloader(folder, ascp_key)
    elif proteins == True:
        refseq_fasta_downloader(folder, ascp_key)
    elif genbank == True:
        refseq_genbank_downloader(folder, ascp_key)
    # ----------------------------

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: Download the latest uniprot protein datasets from Uniprot and
# the accompanying .dat files necessary to create the database.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
import wget
from shutil import copyfileobj
from pathlib import Path
import urllib.request
import gzip

################################################################################
"""---1.0 Define Functions---"""
def uniprot_fasta_downloader(output_file_folder, light):
    # Create protein folder
    merged_db_folder = Path(output_file_folder) / "01.Protein_DB"
    Path(merged_db_folder).mkdir(parents=True, exist_ok=True)
    # Get release number
    release_file = urllib.request.urlopen("ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/complete/reldate.txt")
    release_number = release_file.readline().decode('utf-8').split()[3]
    # Write release information into file
    with open(merged_db_folder / "uniprot_release.txt", 'w') as relinfo:
        relinfo.write("Release number (date) {}".format(release_number))
    # Download fasta files
    print("Downloading fasta files...", flush=True)
    # Overwrite existing temporal files
    if light == True:
        if (merged_db_folder / "uniprot_sprot.fasta.gz").is_file():
            Path.unlink(merged_db_folder / "uniprot_sprot.fasta.gz")
    else:
        if (merged_db_folder / "uniprot_sprot.fasta.gz").is_file():
            Path.unlink(merged_db_folder / "uniprot_sprot.fasta.gz")
        if (merged_db_folder / "uniprot_trembl.fasta.gz").is_file():
            Path.unlink(merged_db_folder / "uniprot_trembl.fasta.gz")
    # Download swissprot and trembl proteins
    if light == True:
        wget.download("ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/complete/uniprot_sprot.fasta.gz",
                  out=str(merged_db_folder / "uniprot_sprot.fasta.gz"))
        compressed = merged_db_folder / "uniprot_sprot.fasta.gz"
        decompressed = merged_db_folder / "uniprot_sprot.fasta"
        with gzip.open(compressed, 'rt') as compressed_fh, open(decompressed, 'w') as decompressed_fh:
            copyfileobj(compressed_fh, decompressed_fh)
            Path.unlink(Path(merged_db_folder / "uniprot_sprot.fasta.gz"))
    else:
        wget.download("ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/complete/uniprot_sprot.fasta.gz",
                  out=str(merged_db_folder / "uniprot_sprot.fasta.gz"))
        wget.download("ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/complete/uniprot_trembl.fasta.gz",
                  out=str(merged_db_folder / "uniprot_trembl.fasta.gz"))
        compressed = merged_db_folder / "uniprot_sprot.fasta.gz"
        decompressed = merged_db_folder / "uniprot_sprot.fasta"
        with gzip.open(compressed, 'rt') as compressed_fh, open(decompressed, 'w') as decompressed_fh:
            copyfileobj(compressed_fh, decompressed_fh)
            Path.unlink(Path(merged_db_folder / "uniprot_sprot.fasta.gz"))
        compressed = merged_db_folder / "uniprot_trembl.fasta.gz"
        decompressed = merged_db_folder / "uniprot_trembl.fasta"
        with gzip.open(compressed, 'rt') as compressed_fh, open(decompressed, 'w') as decompressed_fh:
            copyfileobj(compressed_fh, decompressed_fh)
            Path.unlink(Path(merged_db_folder / "uniprot_trembl.fasta.gz"))
    print("\nDone")
    
def uniprot_dat_downloader(output_file_folder, light):
    # Create dat files folder
    temp_dat_files = Path(output_file_folder) / "02.temp_dat_files"
    Path(temp_dat_files).mkdir(parents=True, exist_ok=True)
    print("Downloading .dat files...", flush=True)
    # Overwrite existing temporal files
    if light == True:
        if (temp_dat_files / "uniprot_sprot.dat.gz").is_file():
            Path.unlink(temp_dat_files / "uniprot_sprot.dat.gz")
    else:
        if (temp_dat_files / "uniprot_sprot.dat.gz").is_file():
            Path.unlink(temp_dat_files / "uniprot_sprot.dat.gz")
        if (temp_dat_files / "uniprot_trembl.dat.gz").is_file():
            Path.unlink(temp_dat_files / "uniprot_trembl.dat.gz")
    # Download swissprot and trembl dat files
    if light == True:
        wget.download("ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/complete/uniprot_sprot.dat.gz",
                    out=str(temp_dat_files / "uniprot_sprot.dat.gz"))
    else:
        wget.download("ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/complete/uniprot_sprot.dat.gz",
                    out=str(temp_dat_files / "uniprot_sprot.dat.gz"))
        wget.download("ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/complete/uniprot_trembl.dat.gz",
                      out=str(temp_dat_files / "uniprot_trembl.dat.gz"))
    print("\nDone")


################################################################################
"""---2.0 Main Function---"""

def main():
    import argparse, sys
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''This script downloads the fasta and .dat files that\n'''
            '''are needed to build the Uniprot annotation database. By default it\n'''
            '''downloads both but you can specify either\n'''
            '''Usage: ''' + sys.argv[0] + ''' -f [output_file folder]\n'''
            '''Global mandatory parameters: -f [output_file folder]\n'''
            '''Optional Database Parameters: See ''' + sys.argv[0] + ' -h')
    parser.add_argument('-f', '--folder', dest='folder', action='store', required=False,
                        help='Folder to store the fasta and genbank files')
    parser.add_argument('--light', dest='light', action='store_true', required=False,
                        help='Light and fast version, only downloads the swissprot database')
    parser.add_argument('--proteins', dest='proteins', action='store_true', required=False,
                        help='Only download the protein fasta files.')
    parser.add_argument('--datfile', dest='datfile', action='store_true', required=False,
                        help='Only download the genbank files')
    args = parser.parse_args()

    folder = args.folder
    proteins = args.proteins
    datfile = args.datfile
    light = args.light

    # ----------------------------
    if proteins == False and datfile == False:
        uniprot_fasta_downloader(folder, light)
        uniprot_dat_downloader(folder, light)
    elif proteins == True:
        uniprot_fasta_downloader(folder, light)
    elif datfile == True:
        uniprot_dat_downloader(folder, light)
    # ----------------------------

if __name__ == "__main__":
    main()


#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0
# Date:         Nov 13, 2020

# Description: Downloads all data required to build the search databases.
# Parses the annotation information and creates the method-specific dbs.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
from independent_scripts import conversion_database_creator
from independent_scripts import uniprot_data_downloader
from independent_scripts import refseq_data_downloader
from independent_scripts import uniprot_dat_parser
from independent_scripts import refseq_genbank_parser
from independent_scripts import sqlite3_database_builder
from independent_scripts import protein_db_creation
from pathlib import Path
from shutil import rmtree


################################################################################
"""---1.0 Define Functions---"""
def db_duilder(output_file_folder, method, light, threads, step, aspera, keep, path=None):
    # Create output_file folder
    outfolder = Path(output_file_folder)
    outfolder.mkdir(parents=True, exist_ok=True)
    genbank_files = None
    database = outfolder / "02.MicrobeAnnotator.db"
    # Download information
    if step == 1:
        print("Downloading UniProt information...")
        uniprot_data_downloader.uniprot_fasta_downloader(outfolder, light)
        uniprot_data_downloader.uniprot_dat_downloader(outfolder, light)
        print("Finished downloading UniProt data!")
        if light == False:
            if aspera == True:
                print("Downloading RefSeq information using Aspera Connect...")
                refseq_data_downloader.refseq_fasta_downloader(outfolder)
                genbank_files = refseq_data_downloader.refseq_genbank_downloader(outfolder)
            else:
                print("Downloading RefSeq information using wget...")
                refseq_data_downloader.refseq_fasta_downloader_wget(outfolder, threads)
                genbank_files = refseq_data_downloader.refseq_genbank_downloader_wget(outfolder, threads)
            print("Finished downloading RefSeq data!")
        step += 1
    # -------------------

    # Parse annotation files
    if step == 2:
        if light == True:
            print("Parsing swissprot annotation data...")
            temporal_sprot_dat = outfolder / "02.temp_dat_files/uniprot_sprot.dat.gz"
            final_sprot_table = outfolder / "02.uniprot_sprot.table"
            uniprot_dat_parser.parse_uniprot_dat(temporal_sprot_dat, final_sprot_table)
            # Remove temporary files
            if keep == False:
                if Path(outfolder / "02.temp_dat_files").is_dir():
                    rmtree(outfolder / "02.temp_dat_files")
            print("Done!")
        else:
            if genbank_files == None and aspera == True:
                temp_gb_files = Path(output_file_folder) / "02.temp_genbank"
                viral_list = refseq_data_downloader.search_all_files(temp_gb_files / "viral")
                bacteria_list = refseq_data_downloader.search_all_files(temp_gb_files / "bacteria")
                archaea_list = refseq_data_downloader.search_all_files(temp_gb_files / "archaea")
                genbank_files = viral_list + bacteria_list + archaea_list
            elif genbank_files == None and aspera == False:
                temp_gb_files = Path(output_file_folder) / "02.temp_genbank"
                genbank_files = refseq_data_downloader.search_all_files(temp_gb_files)
            print("Parsing swissprot and trembl annotation data...")
            temporal_sprot_dat = outfolder / "02.temp_dat_files/uniprot_sprot.dat.gz"
            final_sprot_table = outfolder / "02.uniprot_sprot.table"
            uniprot_dat_parser.parse_uniprot_dat(temporal_sprot_dat, final_sprot_table)
            temporal_trembl_dat = outfolder / "02.temp_dat_files/uniprot_trembl.dat.gz"
            final_trembl_table = outfolder / "02.uniprot_trembl.table"
            uniprot_dat_parser.parse_uniprot_dat(temporal_trembl_dat, final_trembl_table)

            print("Done!")
            print("Parsing refseq annotation data...")
            temp_table_list = refseq_genbank_parser.table_generator_worker(genbank_files, threads)
            final_refseq_table = outfolder / "02.refseq.table"
            refseq_genbank_parser.table_merger(temp_table_list, final_refseq_table, False)
            # Remove temporary files
            if keep == False:
                if Path(outfolder / "02.refseq.table").is_dir():
                    rmtree(outfolder / "02.refseq.table")
                if Path(outfolder / "02.temp_dat_files").is_dir():
                    rmtree(outfolder / "02.temp_dat_files")
            print("Done!")
        step += 1
    # --------------------

    # Merge annotations into SQLite database
    if step == 3:
        print("Building SQLite database: 02.MicrobeAnnotator.db")
        if light == True:
            final_sprot_table = outfolder / "02.uniprot_sprot.table"
            sqlite3_database_builder.create_swissprot_table(str(database), final_sprot_table)
            if keep == False:
                Path(final_sprot_table).unlink()
        else:
            final_sprot_table = outfolder / "02.uniprot_sprot.table"
            final_trembl_table = outfolder / "02.uniprot_trembl.table"
            final_refseq_table = outfolder / "02.refseq.table"
            sqlite3_database_builder.create_swissprot_table(str(database), final_sprot_table)
            sqlite3_database_builder.create_trembl_table(str(database), final_trembl_table)
            sqlite3_database_builder.create_refseq_table(str(database), final_refseq_table)
            if keep == False:
                if Path(final_sprot_table).is_file():
                    Path(final_sprot_table).unlink()
                if Path(final_trembl_table).is_file():
                    Path(final_trembl_table).unlink()
                if Path(final_refseq_table).is_file():
                    Path(final_refseq_table).unlink()
        print("Done!")
        step += 1
    # ---------------------

    # Create interconversion tables
    if step == 4:
        print("Building interconversion databases...")
        interconversion_database = outfolder / "03.Conversion.db"
        print("Building RefSeq to Uniprot")
        try:
            refseq_to_uniprot_table = str(outfolder / "03.uniprot_to_refseq.txt")
            conversion_database_creator.create_refseq_to_uniprot(refseq_to_uniprot_table, interconversion_database, keep)
        except:
            print("Could not create RefSeq to Uniprot table.")
        print("Building KO to EC")
        try:
            conversion_database_creator.create_ko_to_ec(outfolder, interconversion_database, keep)
        except:
            print("Could not create KO to EC table.")
        print("Building Interpro to EC and KO")
        try:
            conversion_database_creator.create_interpro_tables(outfolder, interconversion_database, keep)
        except:
            print("Could not create UniProt tables.")
        step += 1
        print("Done!")
    
    # Create method-specific protein databases
    if step == 5:
        print("Building protein databases...")
        protein_folder = outfolder / "01.Protein_DB"
        if method == "blast":
            protein_db_creation.blastp_db_creator(protein_folder, path)
        elif method == "diamond":
            protein_db_creation.diamond_db_creator(protein_folder, threads, path)
        elif method == "sword":
            protein_db_creation.sword_db_creator(protein_folder, path)
        else:
            print("Please provide a valid db creation method.")
            print("If you are running the microbeannotator_db_builder script this is the last step of the process.")
            print('Given that this last step failed you need to re-run this script starting from --step 4')
            print('and provide the binary folder for the selected method using the "--bin_path" option or add')
            exit('the binary to your PATH (see README for help on this).')
        print("Done!")
        print("---------")
        print("MicrobeAnnotator has finished creating your databases! You are ready to search and annotate your proteins!")


################################################################################
"""---2.0 Main Function---"""

def main():
    import argparse, sys
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''This script build the search databases required by MicrobeAnnotator\n'''
            '''Usage: ''' + sys.argv[0] + ''' -f [output_file folder]\n'''
            '''Global mandatory parameters: -f [output_file folder]\n'''
            '''Optional Database Parameters: See ''' + sys.argv[0] + ' -h')
    parser.add_argument('-d', '--dir', dest='directory', action='store', required=True,
                        help='Directory where database will be created.')
    parser.add_argument('-m', '--method', dest='method', action='store', required=True,
                        help='Search (and db creation) method, one of blast, diamond or sword')
    parser.add_argument('-t', '--threads', dest='threads', action='store', required=False, default=1, type=int,
                        help='Threads to use (when possible). By default 1.')
    parser.add_argument('--bin_path', dest='bin_path', action='store', required=False,
                        help='Path to binary folder for selected method. By defaul assumes the program is in path.')
    parser.add_argument('--step', dest='step', action='store', required=False, default=1, type=int,
                        help='Step to start with. 1.Download data, 2.Parse annotation data, 3.Building SQLite DB, 4.Create interconversion tables, 5.Build protein DBs. Default 1.')
    parser.add_argument('--light', dest='light', action='store_true', required=False,
                        help='Use only KOfamscan and swissprot databases. By default also builds refseq and trembl.')
    parser.add_argument('--no_aspera', dest='aspera', action='store_false', required=False,
                        help='Disables download using Aspera and instead uses wget. By default uses Aspera Connect.')
    parser.add_argument('--keep', dest='keep', action='store_true', required=False,
                        help='Keep intermediate files, can increase disk requirement (not necessary and therefore not recommended)')
    args = parser.parse_args()

    directory = args.directory
    method = args.method
    method = method.lower()
    bin_path = args.bin_path
    light = args.light
    threads = args.threads
    step = args.step
    aspera = args.aspera
    keep = args.keep

    # ----------------------------
    db_duilder(directory, method, light, threads, step, aspera, keep, bin_path)
    # ----------------------------

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: Parses compressed dat files and extracts protein
information relevant for annotation purposes.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
import gzip
from pathlib import Path

################################################################################
"""---1.0 Define Functions---"""

def parse_uniprot_dat(dat_file, output_file_table):
    output_folder = Path(output_file_table).parent
    uniprot_to_refseq = output_folder / "03.uniprot_to_refseq.txt"
    with gzip.open(dat_file, 'rt') as uniprot, open(output_file_table, 'w') as output_file, open(uniprot_to_refseq, 'a') as uni_to_ref:
        gene_id = ""
        accession = ""
        gene_name = ""
        ko_number = ""
        organism = ""
        taxonomy = ""
        function = ""
        compartment = ""
        process = ""
        interpro = ""
        pfam = ""
        ec_number = ""
        refseq_code = ""
        for line in uniprot:
            if line.startswith("ID", 0):
                gene_id = line.split()[1]
            elif "AC  " in line:
                accession = line.split()[1].replace(";","")
            elif "RecName" in line:
                gene_name = line.split("Full=")[1]
                gene_name = gene_name.split("{")[0].strip().replace(";","")
            elif "OS  " in line:
                organism = ' '.join([organism, line.split("OS")[1].strip()]).replace(".","")
            elif "OC  " in line:
                taxonomy = ' '.join([taxonomy, line.split("OC")[1].strip()]).replace(".","")
            elif "DR   KO;" in line:
                ko_number = line.split()[2].replace(";", "")
            elif "DR   GO;" in line:
                if "; F:" in line:
                    code = line.strip().split(";")[1]
                    code = code.strip()
                    if function == "":
                        function = code
                    else:
                        function = ''.join([function, " ", code])
                elif "; C:" in line:
                    code = line.strip().split(";")[1]
                    code = code.strip()
                    if compartment == "":
                        compartment = code
                    else:
                        compartment = ''.join([compartment, " ", code])
                elif "; P:" in line:
                    code = line.strip().split(";")[1]
                    code = code.strip()
                    if process == "":
                        process = code
                    else:
                        process = ''.join([process, " ", code])
            elif "DR   InterPro" in line:
                code = line.strip().split()[2]
                code = code.replace(";", "")
                if interpro == "":
                    interpro = code
                else:
                    interpro = ' '.join([interpro, code])
            elif "DR   Pfam" in line:
                code = line.strip().split()[2]
                code = code.replace(";", "")
                if pfam == "":
                    pfam = code
                else:
                    pfam = ' '.join([pfam, code])
            elif line.startswith("DE") and "EC=" in line:
                ec_code = line.strip().split()[1]
                ec_code = ec_code.replace(";", "")
                ec_code = ec_code.replace("EC=", "")
                if ec_number == "":
                    ec_number = ec_code
                else:
                    ec_number = ' '.join([ec_number, ec_code])
            elif line.startswith("DR") and "RefSeq" in line:
                refseq_code = line.strip().split()[2]
                refseq_code = refseq_code.replace(";", "")
                uni_to_ref.write("{}\t{}\n".format(gene_id, refseq_code))
            elif "//\n" in line:
                if ko_number == "":
                    ko_number = "NA"
                if organism == "":
                    organism = "NA"
                if function == "":
                    function = "NA"
                if compartment == "":
                    compartment = "NA"
                if process == "":
                    process = "NA"
                if interpro == "":
                    interpro = "NA"
                if pfam == "":
                    pfam = "NA"
                if ec_number == "NA":
                    ec_number = "NA"
                output_file.write("{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(gene_id, 
                accession, gene_name, ko_number, organism, taxonomy, function, compartment, process, interpro, pfam, ec_number))
                gene_id = ""
                accession = ""
                gene_name = ""
                ko_number = ""
                organism = ""
                taxonomy = ""
                function = ""
                compartment = ""
                process = ""
                interpro = ""
                pfam =""
                ec_number = ""
                refseq_code = ""


################################################################################
"""---3.0 Main Function---"""

def main():
    import argparse, sys
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(description='''This script parses a Uniprot.dat file and output_files a table with\n'''
                                                    '''the ID, Accession, Gene Name, Organism, Taxonomy, KEGG ID, Function,\n
                                                    Compartment, Process, InterPro, and Pfam\n
                                                    For faster usage in alrge files use gnu parallel (read script file to see how)\n'''
                                    '''\nGlobal mandatory parameters: [Input Uniprot.dat File]\n'''
                                    '''\nOptional Database Parameters: See ''' + sys.argv[0] + ' -h')
    parser.add_argument('-i', '--input', dest='input_dat', action='store', required=True, help='Uniprot.dat file to parse')
    parser.add_argument('-o', '--output_file', dest='output_file_table', action='store', required=False, help='output_file table')
    args = parser.parse_args()

    input_dat = args.input_dat
    output_file_table = args.output_file_table

    parse_uniprot_dat(input_dat, output_file_table)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
########################################################################
# Author:       Carlos A. Ruiz Perez
# Email:        cruizperez3@gatech.edu
# Intitution:   Georgia Institute of Technology
# Version:      1.0.0
# Date:         Nov 13, 2020

# Description: MicrobeAnnotator parses protein fasta files and annotates them
# using several databases in an iterative fashion and summarizes the findings
# using KEGG modules based on KO numbers associated with best database matches.
########################################################################
"""

################################################################################
"""---0.0 Import Modules---"""
from independent_scripts import identifier_conversion
import multiprocessing
from functools import partial
from pathlib import Path
from shutil import rmtree
from shutil import copyfile
from independent_scripts import protein_search
from independent_scripts import fasta_filter_list
from independent_scripts import sqlite3_search
from independent_scripts import ko_mapper
from shutil import which
import pickle

################################################################################
"""---1.0 Main Function---"""

def main():
    import argparse, sys, textwrap
    # Setup parser for arguments.
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
            description='''MicrobeAnnotator parses protein fasta files and annotates them\n'''
            '''using several databases in an iterative fashion and summarizes the findings\n'''
            '''using KEGG modules based on KO numbers associated with best database matches.\n'''
            '''Usage: ''' + sys.argv[0] + ''' -i [protein file] -o [output folder] -d [MicrobeAnnotator db folder]\n'''
            '''-m [search method]\n'''
            '''Global mandatory parameters: -i [protein file] -o [output folder] -d [MicrobeAnnotator db folder]\n'''
            '''-m [search method]\n'''
            '''Optional Database Parameters: See ''' + sys.argv[0] + ' -h')
    general_options = parser.add_argument_group('Mandatory i/o options.')
    general_options.add_argument('-i', '--input', dest='input_list', action='store', required=False, nargs='+',
                        help='Space-separated list of protein files to parse. Use -i OR -l.')
    general_options.add_argument('-l', '--list', dest='file_list', action='store', required=False,
                        help='File with list of inputs. Use -i OR -l.')
    general_options.add_argument('-o', '--outdir', dest='output_dir', action='store', required=True,
                        help='Directory to store results.')
    general_options.add_argument('-d', '--database', dest='database_folder', action='store', required=True,
                        help='Directory where MicrobeAnnotator databases are located.')
    search_options = parser.add_argument_group('Options for search process.')
    search_options.add_argument('-m', '--method', dest='method', action='store', required=True,
                        help='Method used to create databases and to perform seaches. One of "blast", "diamond" or "sword".')
    search_options.add_argument('--kofam_bin', dest='kofam_bin', action='store', required=False, default=None,
                        help='Directory where KOFamscan binaries are located. By default assumes it is in PATH.')
    search_options.add_argument('--method_bin', dest='method_bin', action='store', required=False, default=None,
                        help='Directory where KOFamscan binaries are located. By default assumes it is in PATH.')
    search_options.add_argument('--id_perc', dest='id_perc', action='store', required=False, default=40, type=int,
                        help='Minimum identity percentage to retain a hit. By default 40.')
    search_options.add_argument('--bitscore', dest='bitscore', action='store', required=False, default=80, type=int,
                        help='Minimum bitscore to retain a hit. By default 80.')
    search_options.add_argument('--evalue', dest='evalue', action='store', required=False, default=0.01, type=float,
                        help='Maximum evalue to retain a hit. By default 0.01.')
    search_options.add_argument('--aln_percent', dest='aln_percent', action='store', required=False, default=70, type=int,
                        help='Minimum percentage of query covered by hit alignment. By default 70.')
    plot_options = parser.add_argument_group('Summary and plotting options.')
    plot_options.add_argument('--cluster', dest='cluster', action='store', required=False,
                        help=textwrap.dedent('''
                        Cluster genomes and/or modules. Select "cols" for genomes, "rows" for modules, or "both".
                        By default, no clustering
                        '''))
    plot_options.add_argument('--filename', dest='plot_filename', action='store', required=False, default='metabolic_summary_',
                        help='Prefix for output summary tables and plots. By default "metabolic_summary"')
    misc_options = parser.add_argument_group('Miscellaneous options.')
    misc_options.add_argument('-t', '--threads', dest='threads', action='store', required=False, default=1, type=int,
                        help='Threads to use per processed file, i.e. (per protein file). By default 1.')
    misc_options.add_argument('-p', '--processes', dest='processes', action='store', required=False, default=1, type=int,
                        help=textwrap.dedent('''
                        Number of processes to launch, i.e. number of protein files to process simultaneously.
                        Note this is different from threads. For more information see the README. By default 1.
                        '''))
    misc_options.add_argument('--light', dest='light', action='store_true', required=False,
                        help=textwrap.dedent('''
                        Use only KOfamscan and swissprot databases. By default also uses refseq and
                        trembl (only use if you built both using "microbeannotator_db_builder").
                        '''))
    misc_options.add_argument('--full', dest='full', action='store_true', required=False,
                        help=textwrap.dedent('''
                        Do not perform the iterative annotation but search all proteins against all databases
                        (Increases computation time).
                        '''))
    misc_options.add_argument('--continue_run', dest='continue_run', action='store_true', required=False,
                        help=textwrap.dedent('''
                        If something went wrong when runnin MicrobeAnnotator, try to resume from the last completed step.
                        '''))
    misc_options.add_argument('--refine', dest='refine', action='store_true', required=False,
                        help=textwrap.dedent('''
                        Complement the annotations by finding links to identifiers in other databases.
                        '''))
    args = parser.parse_args()

    input_list = args.input_list
    file_list = args.file_list
    output_dir = args.output_dir
    database_folder = args.database_folder
    method = args.method
    method = method.lower()
    kofam_bin = args.kofam_bin
    method_bin = args.method_bin
    id_perc = args.id_perc
    bitscore = args.bitscore
    evalue = args.evalue
    aln_percent = args.aln_percent
    threads = args.threads
    processes = args.processes
    light = args.light
    cluster = args.cluster
    if cluster != None:
        cluster = cluster.lower()
    plot_filename = args.plot_filename
    full = args.full
    refine = args.refine
    continue_run = args.continue_run

    # Check user input
    if method != 'blast' and method != 'diamond' and method != 'sword':
        exit('Please provide a valid search method with "-m", one of "blast", "diamond" or "sword"')
    print("---- This is MicrobeAnnotator ----")
    if input_list != None:
        print('I will process a total of {} files. Of these, I will run {} in parallel with {} threads per file.'.format(
            str(len(input_list)), str(processes), str(threads)))
    elif file_list != None:
        number_files = 0
        with open(file_list) as file_list_handler:
            for line in file_list_handler:
                number_files += 1
        print('I will process a total of {} files. Of these, I will run {} in parallel with {} threads per file.'.format(
            str(number_files), str(processes), str(threads)))
    if light == True:
        print('For this run I will use KOFamscan and the Swissprot database to annotate your proteins.')
    else:
        print('For this run I will use KOFamscan and the Swissprot, RefSeq and Trembl databases to annotate your proteins.')
    if cluster == None:
        print('When I\'m done I will plot the metabolism summaries using {} as file name prefix using no clustering.'.format(plot_filename))
    elif cluster == 'cols':
        print('When I\'m done I will plot the metabolism summaries using {} as file name prefix and cluster the genomes (cols)'.format(plot_filename))
    elif cluster == 'rows':
        print('When I\'m done I will plot the metabolism summaries using {} as file name prefix and cluster the modules (rows)'.format(plot_filename))
    elif cluster == 'both':
        print('When I\'m done I will plot the metabolism summaries using {} as file name prefix and cluster the genomes and modules'.format(plot_filename))
    else:
        exit('I do not recognize this clustering option: {}.'.format(cluster))
    print("---------\n")

    # Check kofamscan binary existence
    if kofam_bin == None:
        kofam_call = 'exec_annotation'
    else:
        kofam_call = str(Path(kofam_bin) / 'exec_annotation')
    if which(kofam_call) == None:
            if kofam_bin == None:
                exit("KOfamscan not found in PATH, please provide the correct path to the folder with binaries")
            else:
                exit("KOfamscan not found in " + kofam_bin + ", please provide the correct path to the folder with binaries")
    
    # Check search method binary existence
    call_name = None
    if method == 'blast':
        call_name = 'blastp'
    elif method == 'diamond':
        call_name = 'diamond'
    elif method == 'sword':
        call_name = 'sword'
    if method_bin == None:
        method_call = call_name
    else:
        method_call = str(Path(method_bin) / call_name)
    if which(method_call) == None:
            if method_bin == None:
                exit(method_call + " not found in PATH, please provide the correct path to the folder with binaries")
            else:
                exit(method_call + " not found in " + method_bin + ", please provide the correct path to the folder with binaries")
    # ----------------------------

    # Get the initial list of proteins per genome and store them
    if input_list != None and file_list != None:
        exit('Please provide only a file with the list of inputs "-l" OR a space-separated list of files "-i".')
    elif file_list != None:
        input_list = []
        with open(file_list) as file_list_handler:
            for line in file_list_handler:
                input_list.append(line.strip())
    starting_proteins = {}
    for file in input_list:
        starting_filename = str(Path(file).name)
        starting_proteins[starting_filename] = []
        with open(Path(file)) as proteins:
            for line in proteins:
                if line.startswith(">"):
                    line = line.strip().split()[0].replace(">", "")
                    starting_proteins[starting_filename].append(line)
    # ----------------------------

    # Create log folder to track process and folder to store temporal proteins
    process_step = 0
    process_log_folder = Path(output_dir) / "process_log"
    process_log_folder.mkdir(parents=True, exist_ok=True)
    temporal_protein_folder = Path(output_dir) / "temporal_proteins"
    temporal_protein_folder.mkdir(parents=True, exist_ok=True)
    sql_database = str(Path(database_folder) / "02.MicrobeAnnotator.db")
    interconversion_database = str(Path(database_folder) / "03.Conversion.db")
    # Create dictionary with information per protein file
    protein_file_info = {}
    # ----------------------------

    # Check if continue_run was passed, and if so, import the last step completed and the dictionary with information
    if continue_run == True:
        print("Restarting from last checkpoint")
        with open(process_log_folder / "log.txt", 'r') as logfile:
            process_step = int(logfile.readline().strip())
            print(process_step)
        with open(process_log_folder / "structure.pickle", 'rb') as structure_file:
            protein_file_info = pickle.load(structure_file)
    
    if process_step == 0:
        # Search the initial dataset with KOFamscan
        print("Searching proteins using KOFamscan...")
        try:
            pool = multiprocessing.Pool(processes)
            kofam_results = pool.map(partial(protein_search.kofamscan_annotation, 
            multi_argument=(output_dir, threads, kofam_bin)), input_list)
            # Results as (protein_file, protein_file_name, ids_proteins_annotated, final_file)
            # Final annotation fields:
            # * query_id protein_id product ko_number ko_product taxonomy function_go compartment_go process_go interpro pfam ec_number database
        finally:
            pool.close()
            pool.join()
        # ----------------------------

        # Filter out the proteins already annotated
        
        print("Filtering KOFamscan results...")
        # New structure after first step: protein_file : [protein_file_name, final_annotation_file, filtered_fasta_1it]
        for ko_result in kofam_results:
            protein_file_info[ko_result[0]] = [ko_result[1], ko_result[3]]
            outfile = str(temporal_protein_folder / (ko_result[1] + ".1it"))
            # Add first iteration protein files
            protein_file_info[ko_result[0]].append(outfile)
            if full == True:
                copyfile(ko_result[0],outfile)
            else:
                for protein in ko_result[2]:
                    starting_proteins[str(Path(ko_result[0]).name)].remove(protein)
                fasta_filter_list.fastA_filter_list(ko_result[0], outfile, ko_result[2], reverse=True)
        # When the process is complete, write step completed (+1) in the log folder
        # Also, export dictionary with information to be imported in case of continue
        process_step += 1
        with open(process_log_folder / "log.txt", 'w') as logfile:
            logfile.write("{}".format(process_step))
        with open(process_log_folder / "structure.pickle", 'wb') as structure_file:
            pickle.dump(protein_file_info, structure_file)
        # ----------------------------

    if process_step == 1:
        # Search proteins NOT annotated with KOFamscan against Swissprot
        # Determine database to use
        print("Searching proteins against Swissprot using " + method + " ...")
        swissprot_database = None
        if method == "blast":
            swissprot_database = str(Path(database_folder) / "01.Protein_DB/uniprot_sprot")
        elif method == "diamond":
            swissprot_database = str(Path(database_folder) / "01.Protein_DB/uniprot_sprot.dmnd")
        elif method == "sword":
            swissprot_database = str(Path(database_folder) / "01.Protein_DB/uniprot_sprot.fasta")
        input_proteins = []
        for infor in protein_file_info.values():
            input_proteins.append(infor[-1])
        try:
            pool = multiprocessing.Pool(processes)
            arguments_to_pass = (output_dir, 'swissprot', swissprot_database, method,
                                threads, id_perc, bitscore, evalue, aln_percent, method_bin)
            search_results = pool.map(partial(protein_search.similarity_search,
            multiple_arguments=arguments_to_pass), input_proteins)
            # Results as (protein_file, filtered_search_file)
        finally:
            pool.close()
            pool.join()
        # Add name of filtered_search_results to protein_file_info
        # New structure: 
        # protein_file : [protein_file_name, final_annotation_file, filtered_fasta_1it, Swissprot_Search_File]
        temp_dir = protein_file_info.copy()
        for result in search_results:
            for filename, info in temp_dir.items():
                if result[0] == info[2]:
                    protein_file_info[filename].append(result[1])
        del temp_dir

        # Search annotations in SQLite DB and append to the final annotation file
        print("Extracting Swissprot annotation data...")
        for original_file, information in protein_file_info.items():
            filtered_sword_results = information[3]
            final_annotation_file = information[1]
            significant_hits = []
            with open(filtered_sword_results) as swissprot_results:
                for line in swissprot_results:
                    line = line.strip().split()
                    significant_hits.append((line[0], line[1]))
            if len(significant_hits) == 0:
                # If no hits were found
                if light == True:
                    # Write all the proteins without annotation
                    with open(final_annotation_file, 'a') as final_annotation_fh:
                        for original_protein in starting_proteins[str(Path(original_file).name)]:
                            final_annotation_fh.write("{}\tNA\tNo match found\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\n".format(original_protein))
                else:
                    # Copy the first iteration proteins to the second iteration file
                    second_it_outfile = str(temporal_protein_folder / (protein_file_info[original_file][0] + ".2it"))
                    copyfile(protein_file_info[original_file][2],second_it_outfile)
                    protein_file_info[original_file].append(second_it_outfile)
            else:
                # Extract the annotations
                annotation = sqlite3_search.search_ids_imported(sql_database, "swissprot", significant_hits)
                # Each annotation will have:
                # query_id gene_id accession product ko_number organism taxonomy function_GO compartment_GO process_GO interpro_id pfam_id EC_number
                if light == True:
                    # Write the annotations found in the annotation file
                    with open(final_annotation_file, 'a') as final_annotation_fh:
                        for match in annotation:
                            final_annotation_fh.write("{}\t{}\t{}\t{}\tNA\t{}\t{}\t{}\t{}\t{}\t{}\t{}\tswissprot\n".format(match[0],
                            match[1], match[3], match[4], match[6], match[7], match[8], match[9], match[10], match[11], match[12]))
                            if match[4] != "":
                                starting_proteins[str(Path(original_file).name)].remove(match[0])
                    # Check which proteins were not annotated and add information on those
                    # Extract annotated proteins
                    with open(final_annotation_file, 'a') as final_annotation_fh:
                        for original_protein in starting_proteins[str(Path(original_file).name)]:
                            final_annotation_fh.write("{}\tNA\tNo match found\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\n".format(original_protein))
                else:
                    # Write the annotations found in the annotation file
                    with open(final_annotation_file, 'a') as final_annotation_fh:
                        for match in annotation:
                            final_annotation_fh.write("{}\t{}\t{}\t{}\tNA\t{}\t{}\t{}\t{}\t{}\t{}\t{}\tswissprot\n".format(match[0],
                            match[1], match[3], match[4], match[6], match[7], match[8], match[9], match[10], match[11], match[12]))
                            if match[4] != "" and match[4] != "NA":
                                starting_proteins[str(Path(original_file).name)].remove(match[0])
                    # Check which proteins were not annotated and filter for the next iteration
                    second_it_outfile = str(temporal_protein_folder / (protein_file_info[original_file][0] + ".2it"))
                    if full == True:
                        copyfile(protein_file_info[original_file][2],second_it_outfile)
                        protein_file_info[original_file].append(second_it_outfile)
                    else:
                        protein_file_info[original_file].append(second_it_outfile)
                        to_retain = starting_proteins[str(Path(original_file).name)].copy()
                        fasta_filter_list.fastA_filter_list(protein_file_info[original_file][2], 
                        second_it_outfile, to_retain, reverse=False)
                        # starting_proteins[str(Path(original_file).name)]
                    # New structure: 
                    # protein_file : [protein_file_name, final_annotation_file, filtered_fasta_1it, Swissprot_Search_File,
                    # filtered_fasta_2it]
        # When the process is complete, write step completed (+1) in the log folder
        # Also, export dictionary with information to be imported in case of continue
        process_step += 1
        with open(process_log_folder / "log.txt", 'w') as logfile:
            logfile.write("{}".format(process_step))
        with open(process_log_folder / "structure.pickle", 'wb') as structure_file:
            pickle.dump(protein_file_info, structure_file)
        # -----------------------
    
    
    
    # If running complete pipeline search against refseq
    if process_step == 2:
        if light == True:
            print("The light run of MicrobeAnnotator had finished succesfully!")
            print("Summarizing those results...")
            process_step = 5
        else:
            # Determine database to use
            print("Searching proteins against RefSeq...")
            refseq_database = None
            if method == "blast":
                refseq_database = str(Path(database_folder) / "01.Protein_DB/refseq_protein")
            elif method == "diamond":
                refseq_database = str(Path(database_folder) / "01.Protein_DB/refseq_protein.dmnd")
            elif method == "sword":
                refseq_database = str(Path(database_folder) / "01.Protein_DB/refseq_protein.fasta")
            # If we have a second iteration file then perform the searches againts refseq
            input_proteins = []
            for infor in protein_file_info.values():
                if len(infor) == 5:
                    input_proteins.append(infor[-1])
            if len(input_proteins) > 0:
                try:
                    pool = multiprocessing.Pool(processes)
                    arguments_to_pass = (output_dir, 'refseq', refseq_database, method,
                                        threads, id_perc, bitscore, evalue, aln_percent, method_bin)
                    search_results = pool.map(partial(protein_search.similarity_search,
                    multiple_arguments=arguments_to_pass), input_proteins)
                    # Results as (protein_file, filtered_search_file)
                finally:
                    pool.close()
                    pool.join()
            # Add name of filtered_search_results to protein_file_info
            # New structure: 
            # protein_file : [protein_file_name, final_annotation_file, filtered_fasta_1it, Swissprot_Search_File,
            # filtered_fasta_2it, RefSeq_Search_File]
            temp_dir = protein_file_info.copy()
            for result in search_results:
                for filename, info in temp_dir.items():
                    if len(info) == 5 and result[0] == info[4]:
                        protein_file_info[filename].append(result[1])
            del temp_dir
            # --------------------------

            # Search annotations in SQLite DB and append to the final annotation file
            print("Extracting RefSeq annotation data...\n")
            for original_file, information in protein_file_info.items():
                if len(information) == 6:
                    filtered_sword_results = information[5]
                    final_annotation_file = information[1]
                    significant_hits = []
                    with open(filtered_sword_results) as swissprot_results:
                        for line in swissprot_results:
                            line = line.strip().split()
                            significant_hits.append((line[0], line[1]))
                    # If no hits were found
                    if len(significant_hits) == 0:
                        third_it_outfile = str(temporal_protein_folder / (protein_file_info[original_file][0] + ".3it"))
                        copyfile(protein_file_info[original_file][4],third_it_outfile)
                        protein_file_info[original_file].append(third_it_outfile)
                    else:
                        annotation = sqlite3_search.search_ids_imported(sql_database, "refseq", significant_hits)
                        # Each annotation will have:
                        # query_id gene_id product taxonomy ko_number ko_product
                        # Write the annotations found in the annotation file

                        with open(final_annotation_file, 'a') as final_annotation_fh:
                            for match in annotation:
                                final_annotation_fh.write("{}\t{}\t{}\t{}\t{}\t{}\tNA\tNA\tNA\tNA\tNA\t{}\trefseq\n".format(match[0],
                                match[1], match[2], match[5], match[6], match[3], match[4]))
                                if match[5] != "" and match[5] != "NA":
                                    starting_proteins[str(Path(original_file).name)].remove(match[0])
                                
                        # Check which proteins were not annotated and filter for the next iteration
                        third_it_outfile = str(temporal_protein_folder / (protein_file_info[original_file][0] + ".3it"))
                        if full == True:
                            copyfile(protein_file_info[original_file][4],third_it_outfile)
                            protein_file_info[original_file].append(third_it_outfile)
                        else:
                            # Check if all annotations have ko numbers, if not, get ids to next round of iteration
                            protein_file_info[original_file].append(third_it_outfile)
                            to_retain = starting_proteins[str(Path(original_file).name)].copy()
                            fasta_filter_list.fastA_filter_list(protein_file_info[original_file][2], 
                            third_it_outfile, to_retain, reverse=False)
                            # New structure: 
                            # protein_file : [protein_file_name, final_annotation_file, filtered_fasta_1it, Swissprot_Search_File,
                            # filtered_fasta_2it, RefSeq_Search_File, filtered_fasta_3it]
            # When the process is complete, write step completed (+1) in the log folder
            # Also, export dictionary with information to be imported in case of continue  
            process_step += 1
            with open(process_log_folder / "log.txt", 'w') as logfile:
                logfile.write("{}".format(process_step))
            with open(process_log_folder / "structure.pickle", 'wb') as structure_file:
                pickle.dump(protein_file_info, structure_file)
        # --------------------------


        # Finally run remaining proteins against Trembl
    if process_step == 3:
        if light == True:
            print("The light run of MicrobeAnnotator had finished succesfully!")
            print("Summarizing those results...")
            process_step = 5
        else:
            print("Searching proteins against Trembl...\n")
            trembl_database = None
            if method == "blast":
                trembl_database = str(Path(database_folder) / "01.Protein_DB/uniprot_trembl")
            elif method == "diamond":
                trembl_database = str(Path(database_folder) / "01.Protein_DB/uniprot_trembl.dmnd")
            elif method == "sword":
                trembl_database = str(Path(database_folder) / "01.Protein_DB/uniprot_trembl.fasta")
            input_proteins = []
            for infor in protein_file_info.values():
                if len(infor) == 7:
                    input_proteins.append(infor[-1])
            if len(input_proteins) > 0:
                try:
                    pool = multiprocessing.Pool(processes)
                    arguments_to_pass = (output_dir, 'trembl', trembl_database, method,
                                        threads, id_perc, bitscore, evalue, aln_percent, method_bin)
                    search_results = pool.map(partial(protein_search.similarity_search,
                    multiple_arguments=arguments_to_pass), input_proteins)
                    # Results as (protein_file, filtered_search_file)
                finally:
                    pool.close()
                    pool.join()
            # Add name of filtered_search_results to protein_file_info
            # New structure: 
            # protein_file : [protein_file_name, final_annotation_file, filtered_fasta_1it, Swissprot_Search_File,
            # filtered_fasta_2it, RefSeq_Search_File, filtered_fasta_3it, Trembl_Search_File]
            temp_dir = protein_file_info.copy()
            for result in search_results:
                for filename, info in temp_dir.items():
                    if len(info) == 7 and result[0] == info[6]:
                        protein_file_info[filename].append(result[1])
            del temp_dir
            # --------------------------
            # Search annotations in SQLite DB and append to the final annotation file
            print("Extracting Trembl annotation data...\n")
            for original_file, information in protein_file_info.items():
                if len(information) == 8:
                    filtered_sword_results = information[7]
                    final_annotation_file = information[1]
                    significant_hits = []
                    with open(filtered_sword_results) as swissprot_results:
                        for line in swissprot_results:
                            line = line.strip().split()
                            significant_hits.append((line[0], line[1]))
                    if len(significant_hits) == 0:
                        with open(final_annotation_file, 'a') as final_annotation_fh:
                            for original_protein in starting_proteins[str(Path(original_file).name)]:
                                final_annotation_fh.write("{}\tNA\tNo match found\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\n".format(original_protein))
                    else:
                        annotation = sqlite3_search.search_ids_imported(sql_database, "trembl", significant_hits)
                        # Each annotation will have:
                        # query_id gene_id product taxonomy ko_number ko_product
                        with open(final_annotation_file, 'a') as final_annotation_fh:
                            for match in annotation:
                                final_annotation_fh.write("{}\t{}\t{}\t{}\tNA\t{}\t{}\t{}\t{}\t{}\t{}\t{}\ttrembl\n".format(match[0],
                                match[1], match[3], match[4], match[6], match[7], match[8], match[9], match[10], match[11], match[12]))
                                if match[4] != "":
                                    starting_proteins[str(Path(original_file).name)].remove(match[0])
                        with open(final_annotation_file, 'a') as final_annotation_fh:
                            for original_protein in starting_proteins[str(Path(original_file).name)]:
                                final_annotation_fh.write("{}\tNA\tNo match found\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\n".format(original_protein))
            if temporal_protein_folder.is_dir():
                rmtree(temporal_protein_folder)
            # When the process is complete, write step completed (+1) in the log folder
            # Also, export dictionary with information to be imported in case of continue  
            process_step += 1
            with open(process_log_folder / "log.txt", 'w') as logfile:
                logfile.write("{}".format(process_step))
            with open(process_log_folder / "structure.pickle", 'wb') as structure_file:
                pickle.dump(protein_file_info, structure_file)
    # ------------------------

    # Refine annotations with ids from other databases
    if process_step == 4:
        if refine == False:
            process_step += 1
            with open(process_log_folder / "log.txt", 'w') as logfile:
                logfile.write("{}".format(process_step))
            with open(process_log_folder / "structure.pickle", 'wb') as structure_file:
                pickle.dump(protein_file_info, structure_file)
        else:
            import pandas as pd
            print("Improving annotations by searching matches in other databases...")
            # Structure: 
            # protein_file : [protein_file_name, final_annotation_file, filtered_fasta_1it, Swissprot_Search_File,
            # filtered_fasta_2it, RefSeq_Search_File, filtered_fasta_3it, Trembl_Search_File]
            final_annotation_files = []
            for information in protein_file_info.values():
                final_annotation_files.append(str(information[1]))
            
            # Process each annotation file separately
            for annotation_file in final_annotation_files:
                positive_matches = {}
                annotation_table = pd.read_csv(annotation_file, sep="\t", header=0, index_col=0)
                # Parse Uniprot Records
                swissprot_records = annotation_table.loc[(annotation_table['database'] == "swissprot") & (annotation_table['ko_number'] == "NA"), ]
                # Extract records without ko numbers but with ec number
                swissprot_records_ko = swissprot_records.loc[(swissprot_records['ko_number'] == "NA") & (swissprot_records['ec_number'] != "NA"), ]
                swissprot_records_ko_ids = list(swissprot_records_ko['ec_number'])
                if len(swissprot_records_ko_ids) > 0:
                    positive_matches = identifier_conversion.convert_ko_to_ec(swissprot_records_ko_ids, interconversion_database, True)
                    if len(positive_matches) > 0:
                        for identifier, matches in positive_matches.items():
                            annotation_table.loc[(annotation_table['ec_number'] == identifier) & (annotation_table['ko_number'] == "NA"), "ko_number"] = " ".join(matches)
                # Now, extract records without ko number but with interpro ids
                swissprot_records = annotation_table.loc[(annotation_table['database'] == "swissprot") & (annotation_table['ko_number'] == "NA"), ]
                swissprot_records_ko = swissprot_records.loc[(swissprot_records['ko_number'] == "NA") & (swissprot_records['interpro'] != "NA"), ]
                swissprot_records_ko_ids = list(swissprot_records_ko['interpro'])
                if len(swissprot_records_ko_ids) > 0:
                    positive_matches = identifier_conversion.convert_interpro_to_ko(swissprot_records_ko_ids, interconversion_database, False)
                    if len(positive_matches) > 0:
                        for identifier, matches in positive_matches.items():
                            annotation_table.loc[(annotation_table['interpro'] == identifier) & (annotation_table['ko_number'] == "NA"), "ko_number"] = " ".join(matches)
                # Delete used sub-tables
                del swissprot_records
                del swissprot_records_ko
                
                # Parse Trembl Records
                trembl_records = annotation_table.loc[(annotation_table['database'] == "trembl") & (annotation_table['ko_number'] == "NA"), ]
                # Extract records without ko numbers but with ec number
                trembl_records_ko = trembl_records.loc[(trembl_records['ko_number'] == "NA") & (trembl_records['ec_number'] != "NA"), ]
                trembl_records_ko_ids = list(trembl_records_ko['ec_number'])
                if len(trembl_records_ko_ids) > 0:
                    positive_matches = identifier_conversion.convert_ko_to_ec(trembl_records_ko_ids, interconversion_database, True)
                    if len(positive_matches) > 0:
                        for identifier, matches in positive_matches.items():
                            annotation_table.loc[(annotation_table['ec_number'] == identifier) & (annotation_table['ko_number'] == "NA"), "ko_number"] = " ".join(matches)
                # Now, extract records without ko number but with interpro ids
                trembl_records = annotation_table.loc[(annotation_table['database'] == "trembl") & (annotation_table['ko_number'] == "NA"), ]
                trembl_records_ko = trembl_records.loc[(trembl_records['ko_number'] == "NA") & (trembl_records['interpro'] != "NA"), ]
                trembl_records_ko_ids = list(trembl_records_ko['interpro'])
                if len(trembl_records_ko_ids) > 0:
                    positive_matches = identifier_conversion.convert_interpro_to_ko(trembl_records_ko_ids, interconversion_database, False)
                    if len(positive_matches) > 0:
                        for identifier, matches in positive_matches.items():
                            annotation_table.loc[(annotation_table['interpro'] == identifier) & (annotation_table['ko_number'] == "NA"), "ko_number"] = " ".join(matches)
                del trembl_records
                del trembl_records_ko

                # Parse RefSeq Records
                refseq_records = annotation_table.loc[(annotation_table.database == "refseq") & (annotation_table['ko_number'] == "NA"), ]
                # Extract records without ko numbers but with ec number
                refseq_records_ko = refseq_records.loc[(refseq_records['ko_number'] == "NA") & (refseq_records['ec_number'] != "NA"), ]
                refseq_records_ko_ids = list(refseq_records_ko['ec_number'])
                if len(refseq_records_ko_ids) > 0:
                    positive_matches = identifier_conversion.convert_ko_to_ec(refseq_records_ko_ids, interconversion_database, True)
                    if len(positive_matches) > 0:
                        for identifier, matches in positive_matches.items():
                            annotation_table.loc[(annotation_table['ec_number'] == identifier), "ko_number"] = " ".join(matches)
                # Extract refseq ids for records with no ko
                refseq_records = annotation_table.loc[(annotation_table.database == "refseq") & (annotation_table['ko_number'] == "NA"), ]
                refseq_records_ids = list(refseq_records['protein_id'])
                if len(refseq_records_ids) > 0:
                    refseq_records_indices = refseq_records.index
                    query_refseq_pair = list(zip(refseq_records_indices, refseq_records_ids))
                    # Convert refseq to uniprot
                    refseq_uniprot = {}
                    positive_matches = identifier_conversion.convert_refseq_to_uniprot(refseq_records_ids, interconversion_database, False)
                    if len(positive_matches) > 0:
                        for identifier, matches in positive_matches.items():
                            refseq_uniprot[identifier] = matches[0]
                        query_uniprot_pair = []
                        for pair in query_refseq_pair:
                            if pair[1] in refseq_uniprot:
                                query_uniprot_pair.append((pair[refseq_records_indices], refseq_uniprot[pair[1]]))
                        # Now estract the uniprot annotations using the identifiers found in the previous step
                        uniprot_annotations = sqlite3_search.search_ids_imported(sql_database, "trembl", query_uniprot_pair)
                        if len(uniprot_annotations) > 0:
                            # Parse results and modify the original table
                            for annotation in uniprot_annotations:
                                original_index = annotation.pop(0)
                                annotation_table.loc[original_index] = annotation
                # Save the refined table in the original location
                pd.DataFrame.to_csv(annotation_table, sep="\t", index=0, header=True)
            # When the process is complete, write step completed (+1) in the log folder
            # Also, export dictionary with information to be imported in case of continue
            process_step += 1
            with open(process_log_folder / "log.txt", 'w') as logfile:
                logfile.write("{}".format(process_step))
            with open(process_log_folder / "structure.pickle", 'wb') as structure_file:
                pickle.dump(protein_file_info, structure_file)
            print("Done!")
    # ------------------------

    # Parse annotation files and summarize them
    if process_step == 5:
        print("Extracting ko numbers and summarizing results...")
        annotation_files = []
        for information in protein_file_info.values():
            annotation_folder = information[1].parent
            ko_numbers = str(annotation_folder / (information[0] + ".ko"))
            with open(information[1], 'r') as annotations, open(ko_numbers, 'w') as ko_present:
                for line in annotations:
                    line = line.strip().split("\t")
                    # If there are multiple KOs
                    if line[3] != "":
                        if len(line[3].split()) > 1:
                            for element in line[3].split():
                                ko_present.write("{}\n".format(element))
                        else:
                            ko_present.write("{}\n".format(line[3]))
            annotation_files.append(ko_numbers)
        prefix = str(Path(output_dir) / plot_filename)
        regular_modules, bifurcation_modules, structural_modules,  \
        module_information, metabolism_matrix, module_group_matrix = ko_mapper.module_information_importer(annotation_files)
        metabolic_annotation = ko_mapper.global_mapper(regular_modules, bifurcation_modules, structural_modules, annotation_files)
        metabolism_matrix_dropped_relabel, module_colors = ko_mapper.create_output_files(metabolic_annotation, metabolism_matrix, module_information, cluster, prefix)
        ko_mapper.plot_function_barplots(module_colors, module_group_matrix, metabolism_matrix_dropped_relabel, prefix)
        print("MicrobeAnnotator has finished succefully!")
    # ------------------------

if __name__ == "__main__":
    main()




from setuptools import setup, find_packages

setup(
    name='microbeannotator',
    version='1.0.1',
    description='A user friendly microbe genome annotation tool',
    url='https://github.com/cruizperez/MicrobeAnnotator',
    author='Carlos A. Ruiz Perez',
    author_email='cruizperez3@gatech.edu',
    packages=find_packages(),
    include_package_data=True,
    use_scm_version=True,
    setup_requires=[
        'setuptools_scm'],
    python_requires='>=3.5, <3.9',
    scripts=['microbeannotator.py',
             'microbeannotator_db_builder.py', 
             'independent_scripts/identifier_conversion.py',
             'independent_scripts/ko_mapper.py'],
    entry_points={
        'console_scripts': [
            'microbeannotator=microbeannotator:main',
            'microbeannotator_db_builder=microbeannotator_db_builder:main',
        ],
    },
)
